#!/usr/bin/env python

import logging
import time
import datetime
from .HTMLElement import HTMLElement
from .attr_property import attr_property
from .compatibility import thug_long

log = logging.getLogger("Thug")


class HTMLAnchorElement(HTMLElement):
    def __init__(self, doc, tag):
        HTMLElement.__init__(self, doc, tag)

    accessKey       = attr_property("accesskey")
    charset         = attr_property("charset", default = "")
    coords          = attr_property("coords")
    href            = attr_property("href")
    hreflang        = attr_property("hreflang")
    name            = attr_property("name")
    rel             = attr_property("rel")
    rev             = attr_property("rev")
    shape           = attr_property("shape")
    tabIndex        = attr_property("tabindex", thug_long)
    target          = attr_property("target")
    type            = attr_property("type")

    def blur(self):
        pass

    def focus(self):
        pass

    def click(self):
        now = datetime.datetime.now()
        self.tag['_clicked'] = time.mktime(now.timetuple())
        if self.href:
            log.DFT.follow_href(self.href)



#!/usr/bin/env python
#
# CCInterpreter.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA

import logging
log = logging.getLogger("Thug")


class CCInterpreter(object):
    """
        Microsoft Internet Explorer Conditional Comments tiny interpreter
    """
    def __init__(self):
        pass

    def run(self, script):
        script = script.replace('@cc_on!@', '*/!/*')

        if '/*@cc_on' in script:
            script = script.replace('/*@cc_on', '')
            script = script.replace('@_jscript_version', str(log.ThugOpts.Personality.cc_on['_jscript_version']))
            script = script.replace('/*@if', 'if')
            script = script.replace('@if', 'if')
            script = script.replace('@elif', 'else if')
            script = script.replace('@else', 'else')
            script = script.replace('/*@end', '')
            script = script.replace('@end', '')
            script = script.replace('@_alpha', 'false')
            script = script.replace('@_mc680x0', 'false')
            script = script.replace('@_win16', 'false')
            script = script.replace('@_win64', 'false')
            script = script.replace('@_x86', 'true')

            if log.ThugOpts.Personality.platform in ('Win32', ):
                script = script.replace('@_win32', 'true')
                script = script.replace('@_mac', 'false')
            if log.ThugOpts.Personality.platform in ('MacIntel', ):
                script = script.replace('@_win32', 'false')
                script = script.replace('@_mac', 'true')

            script = script.replace('@*/', '')
            script = script.replace('/*@', '')

        return script



# AOL Radio AOLMediaPlaybackControl.exe
# CVE-2007-6250

import logging

log = logging.getLogger("Thug")


def AppendFileToPlayList(self, arg):
    if len(arg) > 512:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "AOL Radio AOLMediaPlaybackControl ActiveX",
                                          "Overflow in AppendFileToPlayList",
                                          cve = 'CVE-2007-6250')

        log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2007-6250", None)


def ConvertFile(self, *arg):
    # FIXME
    if len(arg[0]) > 512:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "AOL Radio AOLMediaPlaybackControl ActiveX",
                                          "Overflow in ConvertFile",
                                          cve = 'CVE-2007-6250')

        log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2007-6250", None)



# GlobalLink ConnectAndEnterRoom ActiveX Control ConnectAndEnterRoom() Method Overflow Vulnerability
# CVE-2007-5722

import logging

log = logging.getLogger("Thug")


def ConnectAndEnterRoom(self, arg0, arg1, arg2, arg3, arg4, arg5):
    if len(arg0) > 172:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "GlobalLink ConnectAndEnterRoom ActiveX",
                                          "ConnectAndEnterRoom Overflow",
                                          cve = 'CVE-2007-5722')

        log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2007-5722", None)



#!/usr/bin/env python

from .HTMLElement import HTMLElement
from .attr_property import attr_property
from .text_property import text_property


class HTMLScriptElement(HTMLElement):
    def __init__(self, doc, tag):
        HTMLElement.__init__(self, doc, tag)

    text            = text_property()
    htmlFor         = None
    event           = None
    charset         = attr_property("charset", default = "")
    defer           = attr_property("defer", bool)
    src             = attr_property("src", default = "")
    type            = attr_property("type")




class XMLDOMParseError(object):
    def __init__(self):
        self._errorCode = 0
        self._filepos   = 0
        self._line      = 0
        self._linepos   = 0
        self._reason    = 0
        self._srcText   = ''
        self._url       = ''

    @property
    def errorCode(self):  # pylint:disable=unused-variable
        return self._errorCode

    @property
    def filepos(self):  # pylint:disable=unused-variable
        return self._filepos

    @property
    def line(self):  # pylint:disable=unused-variable
        return self._line

    @property
    def linepos(self):  # pylint:disable=unused-variable
        return self._linepos

    @property
    def reason(self):  # pylint:disable=unused-variable
        return self._reason

    @property
    def srcText(self):  # pylint:disable=unused-variable
        return self._srcText

    @property
    def url(self):  # pylint:disable=unused-variable
        return self._url



#!/usr/bin/env python

from .HTMLElement import HTMLElement
from .attr_property import attr_property


class HTMLFrameSetElement(HTMLElement):
    def __init__(self, doc, tag):
        HTMLElement.__init__(self, doc, tag)

    cols            = attr_property("cols")
    rows            = attr_property("rows")



#!/usr/bin/env python

from thug.DOM.JSClass import JSClass


# Introduced in DOM Level 2
class EventException(RuntimeError, JSClass):
    def __init__(self, code):
        self.code = code

    # Exception Code
    UNSPECIFIED_EVENT_TYPE_ERR = 0  # If the Event's type was not specified by initializing the event before the
                                    # method was called. Specification of the Event's type as null or an empty
                                    # string will also trigger this exception.




import logging

log = logging.getLogger("Thug")


def Seturl(self, val):
    self.__dict__['url'] = val

    log.ThugLogging.log_exploit_event(self._window.url,
                                      "RediffBolDownloader ActiveX",
                                      "Overflow in url property")



#!/usr/bin/env python
#
# Plugin.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA


class Plugin(dict):
    """A dictionary with attribute-style access. It maps attribute access to
    the real dictionary.  """
    def __init__(self, init = None):
        if init is None:
            init = dict()

        dict.__init__(self, init)

    def __getstate__(self):
        return list(self.__dict__.items())

    def __setstate__(self, items):
        for key, val in items:
            self.__dict__[key] = val

    def __repr__(self):
        return "%s(%s)" % (self.__class__.__name__, dict.__repr__(self))

    def __setitem__(self, key, value):
        return super(Plugin, self).__setitem__(key, value)

    def __getitem__(self, name):
        return super(Plugin, self).__getitem__(name)

    def __delitem__(self, name):
        return super(Plugin, self).__delitem__(name)

    __getattr__ = __getitem__
    __setattr__ = __setitem__



# Kingsoft Antivirus
# CVE-NOMATCH

import logging

log = logging.getLogger("Thug")


def SetUninstallName(self, arg):
    if len(arg) > 900:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "Kingsoft AntiVirus ActiveX",
                                          "SetUninstallName Heap Overflow")



# RTS Sentry Digital Surveillance PTZCamPanel Class (CamPanel.dll 2.1.0.2)
# CVE-NOMATCH

import logging

log = logging.getLogger("Thug")


def ConnectServer(self, server, user):
    if len(user) > 1024:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "PTZCamPanel ActiveX",
                                          "Overflow in ConnectServer user arg")



#!/usr/bin/env python

from .HTMLElement import HTMLElement
from .attr_property import attr_property
from .text_property import text_property
from .compatibility import thug_long


class HTMLOptionElement(HTMLElement):
    def __init__(self, doc, tag):
        HTMLElement.__init__(self, doc, tag)

    @property
    def form(self):
        raise NotImplementedError()

    defaultSelected = attr_property("selected", bool)
    text            = text_property(readonly = True)
    index           = attr_property("index", thug_long, readonly = True)
    disabled        = attr_property("disabled", bool)
    label           = attr_property("label")
    selected        = False
    value           = attr_property("value")



#!/usr/bin/env python

from thug.DOM.JSClass import JSClass


class NamedNodeMap(JSClass):
    def __init__(self, parent):
        self.parent = parent

    def getNamedItem(self, name):
        return self.parent.getAttributeNode(name)

    def __getattr__(self, name):
        return self.getNamedItem(name)

    def setNamedItem(self, attr):
        oldattr = self.parent.getAttributeNode(attr.name)

        attr.parent = self.parent

        self.parent.tag[attr.name] = attr.value

        if oldattr:
            oldattr.parent = None

        return oldattr

    def removeNamedItem(self, name):
        self.parent.removeAttribute(name)

    def item(self, index):
        names = list(self.parent.tag.attrMap.keys())
        return self.parent.getAttributeNode(names[index]) if 0 <= index and index < len(names) else None

    @property
    def length(self):
        return len(self.parent.tag._getAttrMap())



#!/usr/bin/env python
#
# External.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA

from .JSClass import JSClass

import logging
log = logging.getLogger("Thug")


class External(JSClass):
    def __init__(self):
        self._providers = set()
        self._channels  = set()
        self._favorites = set()

        self.__init_personality()

    def __init_personality(self):
        if log.ThugOpts.Personality.isIE():
            self.__init_personality_IE()
            return

        if log.ThugOpts.Personality.isChrome():
            self.__init_personality_Chrome()

    def __init_personality_IE(self):
        self.frozen               = self._frozen
        self.menuArguments        = self._menuArguments
        self.AddDesktopComponent  = self._AddDesktopComponent
        self.AddFavorite          = self._AddFavorite
        self.AutoCompleteSaveForm = self._AutoCompleteSaveForm
        self.AutoScan             = self._AutoScan
        self.bubbleEvent          = self._bubbleEvent
        self.IsSubscribed         = self._IsSubscribed
        self.NavigateAndFind      = self._NavigateAndFind
        self.raiseEvent           = self._raiseEvent
        self.ShowBrowserUI        = self._ShowBrowserUI

        if log.ThugOpts.Personality.browserMajorVersion < 7:
            self.AddChannel = self._AddChannel

        if log.ThugOpts.Personality.browserMajorVersion >= 7:
            self.AddSearchProvider         = self._AddSearchProvider
            self.IsSearchProviderInstalled = self._IsSearchProviderInstalled

    def __init_personality_Chrome(self):
        self.AddSearchProvider = self._AddSearchProvider

    @property
    def _frozen(self):
        return False

    @property
    def _menuArguments(self):
        return None

    def _AddChannel(self, URL):
        self._channels.add(URL)

    def _AddDesktopComponent(self, URL, type, left = None, top = None, width = None, height = None):  # pylint:disable=redefined-builtin
        pass

    def _AddFavorite(self, URL, title = None):
        self._favorites.add((URL, title))

    def _AddSearchProvider(self, URL):
        self._providers.add(URL)

    def _AutoCompleteSaveForm(self, formElement):
        pass

    def _AutoScan(self, domainPart, defaultURL = None, target = None):
        # This method does not work in Internet Explorer from version 7
        # and raises an exception.
        if log.ThugOpts.Personality.browserMajorVersion >= 7:
            raise TypeError()

    def _bubbleEvent(self):
        pass

    def _IsSearchProviderInstalled(self, URL):
        if URL in self._providers:
            return 1  # A matching search provider is installed, but it is not the default.

        return 0    # No installed search provider was found with the specified prefix

    def _IsSubscribed(self, URL):
        return False

    def _NavigateAndFind(self, URL, textToFind, findInFrame):
        pass

    def _raiseEvent(self, eventName, eventObj):
        pass

    def _ShowBrowserUI(dialogBoxType, null = None):
        pass



# DivX Player 6.6.0 ActiveX Control
# CVE-NOMATCH

import logging

log = logging.getLogger("Thug")


def SetPassword(self, arg0):
    if len(arg0) > 128:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "DivX Player ActiveX",
                                          "Overflow in SetPassword")



#!/usr/bin/env python

import logging

log = logging.getLogger("Thug")


# Introduced in DOM Level 2
class DocumentView(object):
    def __init__(self, doc):
        self.doc = doc

    @property
    def defaultView(self):
        # return None
        return getattr(self, 'window', None)



# Symantec AppStream LaunchObj ActiveX Arbitrary File Download and Execute
# CVE-2008-4388

import logging

log = logging.getLogger("Thug")


def installAppMgr(self, url):
    log.ThugLogging.log_exploit_event(self._window.url,
                                      "Symantec AppStream LaunchObj ActiveX",
                                      "Arbitrary File Download and Execute",
                                      cve = "CVE-2008-4388",
                                      data = {
                                                "url": url
                                             }
                                     )

    log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2008-4388", None)
    log.ThugLogging.add_behavior_warn("[Symantec AppStream LaunchObj ActiveX] Fetching from URL %s" % (url, ))

    try:
        self._window._navigator.fetch(url, redirect_type = "CVE-2008-4388")
    except:  # pylint:disable=bare-except
        log.ThugLogging.add_behavior_warn('[Symantec AppStream LaunchObj ActiveX] Fetch failed')



#!/usr/bin/env python

import copy
import bs4 as BeautifulSoup
import logging

from .abstractmethod import abstractmethod
from .DOMException import DOMException
from .Events.EventTarget import EventTarget
from .NodeList import NodeList

log = logging.getLogger("Thug")

from thug.DOM.JSClass import JSClass


class Node(JSClass, EventTarget):
    # NodeType
    ELEMENT_NODE                   = 1
    ATTRIBUTE_NODE                 = 2
    TEXT_NODE                      = 3
    CDATA_SECTION_NODE             = 4
    ENTITY_REFERENCE_NODE          = 5
    ENTITY_NODE                    = 6
    PROCESSING_INSTRUCTION_NODE    = 7
    COMMENT_NODE                   = 8
    DOCUMENT_NODE                  = 9
    DOCUMENT_TYPE_NODE             = 10
    DOCUMENT_FRAGMENT_NODE         = 11
    NOTATION_NODE                  = 12

    def __init__(self, doc):
        self.doc = doc
        EventTarget.__init__(self)
        self.__init_personality()

    def __repr__(self):
        return "<Node %s at 0x%08X>" % (self.nodeName, id(self))

    def __eq__(self, other):
        return hasattr(other, "doc") and self.doc == other.doc

    def __ne__(self, other):
        return not self.__eq__(other)

    def __init_personality(self):
        if log.ThugOpts.Personality.isIE():
            self.__init_personality_IE()
            return

        if log.ThugOpts.Personality.isFirefox():
            self.__init_personality_Firefox()
            return

        if log.ThugOpts.Personality.isChrome():
            self.__init_personality_Chrome()
            return

        if log.ThugOpts.Personality.isSafari():
            self.__init_personality_Safari()
            return

        if log.ThugOpts.Personality.isOpera():
            self.__init_personality_Opera()

    def __init_personality_IE(self):
        self.applyElement = self._applyElement

        # Internet Explorer < 9 does not implement compareDocumentPosition
        if log.ThugOpts.Personality.browserMajorVersion >= 9:
            self.compareDocumentPosition = self._compareDocumentPosition

    def __init_personality_Firefox(self):
        self.compareDocumentPosition = self._compareDocumentPosition

    def __init_personality_Chrome(self):
        self.compareDocumentPosition = self._compareDocumentPosition

    def __init_personality_Safari(self):
        self.compareDocumentPosition = self._compareDocumentPosition

    def __init_personality_Opera(self):
        self.compareDocumentPosition = self._compareDocumentPosition

    @property
    @abstractmethod
    def nodeType(self):
        pass

    @property
    @abstractmethod
    def nodeName(self):
        pass

    @abstractmethod
    def getNodeValue(self):
        return None

    @abstractmethod
    def setNodeValue(self, value):
        raise DOMException(DOMException.NO_MODIFICATION_ALLOWED_ERR)

    nodeValue = property(getNodeValue, setNodeValue)

    @property
    def attributes(self):
        return None

    @property
    def childNodes(self):
        return NodeList(self.doc, [])

    @property
    def firstChild(self):
        return None

    @property
    def lastChild(self):
        return None

    @property
    def nextSibling(self):
        return None

    @property
    def previousSibling(self):
        return None

    @property
    def parentNode(self):
        return None

    # Introduced in DOM Level 2
    @property
    def namespaceURI(self):
        return None

    # Introduced in DOM Level 2
    @property
    def prefix(self):
        return None

    # Introduced in DOM Level 2
    @property
    def localName(self):
        return None

    # Modified in DOM Level 2
    @property
    def ownerDocument(self):
        return log.DFT.window.doc
        # return self.doc

    def findChild(self, child):
        # try:
        #    return self.tag.contents.index(child.tag)
        # except:
        #    return -1
        if getattr(child, 'tag', None) and child.tag in self.tag.contents:
            childHash = hash(child.tag._node)

            for p in self.tag.contents:
                if getattr(p, '_node', None) is None:
                    continue

                if childHash == hash(p._node):
                    return self.tag.contents.index(p)

        return -1

    def is_readonly(self, node):
        return node.nodeType in (Node.DOCUMENT_TYPE_NODE,
                                 Node.NOTATION_NODE,
                                 Node.ENTITY_REFERENCE_NODE,
                                 Node.ENTITY_NODE, )

    def is_text(self, node):
        return node.nodeType in (Node.TEXT_NODE,
                                 Node.PROCESSING_INSTRUCTION_NODE,
                                 Node.CDATA_SECTION_NODE,
                                 Node.COMMENT_NODE, )

    def insertBefore(self, newChild, refChild):
        if not newChild:
            raise DOMException(DOMException.HIERARCHY_REQUEST_ERR)

        if not isinstance(newChild, Node):
            raise DOMException(DOMException.HIERARCHY_REQUEST_ERR)

        # If refChild is null, insert newChild at the end of the list
        # of children
        if not refChild:
            return self.appendChild(newChild)

        if not isinstance(refChild, Node):
            raise DOMException(DOMException.HIERARCHY_REQUEST_ERR)

        # index = self.findChild(refChild)
        # if index < 0 and not self.is_text(refChild):
        #    raise DOMException(DOMException.NOT_FOUND_ERR)

        # If the newChild is already in the tree, it is first removed
        if getattr(newChild, 'tag', None) and newChild.tag in self.tag.contents:
            newChildHash = hash(newChild.tag._node)

            for p in self.tag.contents:
                if getattr(p, '_node', None) is None:
                    continue

                if newChildHash == hash(p._node):
                    p.extract()

            # self.tag.contents.remove(newChild.tag)

        index = self.findChild(refChild)
        if index < 0 and not self.is_text(refChild):
            raise DOMException(DOMException.NOT_FOUND_ERR)

        if self.is_text(newChild):
            self.tag.insert(index, newChild.data.output_ready(formatter = lambda x: x))
            return newChild

        if newChild.nodeType in (Node.DOCUMENT_FRAGMENT_NODE, ):
            # self.tag.insert(index, newChild.tag.findChild())
            node = None

            for p in newChild.tag.find_all_next():
                if node is None:
                    self.tag.insert(index, p)
                else:
                    node.append(p)

                node = p

            return newChild

        self.tag.insert(index, newChild.tag)
        return newChild

    def replaceChild(self, newChild, oldChild):
        # NO_MODIFICATION_ALLOWED_ERR: Raised if this node or the parent of
        # the new node is readonly.
        if self.is_readonly(self):
            raise DOMException(DOMException.NO_MODIFICATION_ALLOWED)

        parent = getattr(newChild, 'parentNode', None)
        if parent:
            if self.is_readonly(parent):
                raise DOMException(DOMException.NO_MODIFICATION_ALLOWED)

        if not newChild or not oldChild:
            raise DOMException(DOMException.HIERARCHY_REQUEST_ERR)

        if not isinstance(newChild, Node) or not isinstance(oldChild, Node):
            raise DOMException(DOMException.HIERARCHY_REQUEST_ERR)

        index = self.findChild(oldChild)
        if index < 0:
            raise DOMException(DOMException.NOT_FOUND_ERR)

        if self.is_text(newChild):
            self.tag.contents[index] = newChild.data.output_ready(formatter = lambda x: x)
            return oldChild

        if newChild.nodeType in (Node.DOCUMENT_FRAGMENT_NODE, ):
            # self.tag.contents[index] = newChild.tag.findChild()
            node = None

            for p in newChild.tag.find_all_next():
                if node is None:
                    self.tag.contents[index] = p
                else:
                    node.append(p)

                node = p

            return oldChild

        self.tag.contents[index] = newChild.tag
        return oldChild

    def removeChild(self, oldChild):
        # NO_MODIFICATION_ALLOWED_ERR: Raised if this node is readonly
        if self.is_readonly(self):
            raise DOMException(DOMException.NO_MODIFICATION_ALLOWED)

        if not oldChild:
            raise DOMException(DOMException.NOT_FOUND_ERR)

        if not isinstance(oldChild, Node):
            raise DOMException(DOMException.NOT_FOUND_ERR)

        index = self.findChild(oldChild)
        if index < 0 and not self.is_text(oldChild):
            raise DOMException(DOMException.NOT_FOUND_ERR)

        if getattr(oldChild, 'tag', None) and oldChild.tag in self.tag.contents:
            oldChildHash = hash(oldChild.tag._node)

            for p in self.tag.contents:
                if getattr(p, '_node', None) is None:
                    continue

                if oldChildHash == hash(p._node):
                    p.extract()
            # self.tag.contents.remove(oldChild.tag)

        return oldChild

    def appendChild(self, newChild):
        # NO_MODIFICATION_ALLOWED_ERR: Raised if this node is readonly
        if self.is_readonly(self):
            raise DOMException(DOMException.NO_MODIFICATION_ALLOWED)

        if self.is_text(self):
            raise DOMException(DOMException.HIERARCHY_REQUEST_ERR)

        if not newChild:
            raise DOMException(DOMException.HIERARCHY_REQUEST_ERR)

        if not isinstance(newChild, Node):
            raise DOMException(DOMException.HIERARCHY_REQUEST_ERR)

        # If the newChild is already in the tree, it is first removed
        if getattr(newChild, 'tag', None) and newChild.tag in self.tag.contents:
            newChildHash = hash(newChild.tag._node)

            for p in self.tag.contents:
                if getattr(p, '_node', None) is None:
                    continue

                if newChildHash == hash(p._node):
                    p.extract()
            # self.tag.contents.remove(newChild.tag)

        if self.is_text(newChild):
            self.tag.append(newChild.data.output_ready(formatter = lambda x: x))
            return newChild

        if newChild.nodeType in (Node.DOCUMENT_FRAGMENT_NODE, ):
            # self.tag.append(newChild.tag.findChild())
            node = self.tag
            for p in newChild.tag.find_all_next():
                node.append(p)
                node = p

            return newChild

        self.tag.append(newChild.tag)
        return newChild

    def hasChildNodes(self):
        return len(self.tag.contents) > 0

    def _applyElement(self, element, where = 'inside'):
        where = where.lower()

        if where in ('inside', ):
            self.appendChild(element)

        if where in ('outside', ):
            self.insertBefore(element, self)

    # Modified in DOM Level 2
    def normalize(self):
        pass

    # Introduced in DOM Level 2
    def isSupported(self, feature, version):
        from .DOMImplementation import DOMImplementation
        return DOMImplementation.hasFeature(feature, version)

    # Introduced in DOM Level 2
    def hasAttributes(self):
        return False

    # Introduced in DOM Level 3
    def _compareDocumentPosition(self, node):
        return None

    # @abstractmethod
    def cloneNode(self, deep):
        # Returns a duplicate of this node
        cloned = copy.copy(self)

        # The duplicate node has no parent (parentNode is null)
        cloned.tag.parent = None

        # Cloning an Element copies all attributes and their values but
        # this method does not copy any text it contains unless it is a
        # deep clone, since the Text is contained in a child Text node.
        if cloned.nodeType in (Node.ELEMENT_NODE, ) and deep is False:
            cloned.tag.string = ''

        # return cloned
        return self.wrap(self.doc, cloned.tag)

    @staticmethod
    def wrap(doc, obj):
        from .Element import Element

        if obj is None:
            return None

        if type(obj) == BeautifulSoup.CData:
            from .CDATASection import CDATASection
            return CDATASection(doc, obj)

        if type(obj) == BeautifulSoup.NavigableString:
            from .Text import Text
            return Text(doc, obj)

        return Element(doc, obj)



# Xunlei Thunder PPLAYER.DLL_1.WORK ActiveX Control

import logging

log = logging.getLogger("Thug")


def DownURL2(self, arg0, arg1, arg2, arg3):
    if len(arg0) > 1024:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "Xunlei Thunder PPlayer ActiveX",
                                          "DownURL2 Overflow")


def SetFlvPlayerUrl(self, val):
    self.__dict__['FlvPlayerUrl'] = val

    if len(val) > 1060:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "Xunlei Thunder PPlayer ActiveX",
                                          "FlvPlayerUrl Property Handling Buffer Overflow")


def SetLogo(self, val):
    self.__dict__['Logo'] = val

    if len(val) > 128:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "Xunlei Thunder PPlayer ActiveX",
                                          "Remote Overflow Exploit in Logo property")



#!/usr/bin/env python

from .HTMLElement import HTMLElement
from .text_property import text_property


class HTMLTitleElement(HTMLElement):
    def __init__(self, doc, tag):
        HTMLElement.__init__(self, doc, tag)

    text            = text_property()



# Microsoft XMLDOM
from lxml import etree

import base64
import binascii
from thug.DOM.W3C import w3c
from thug.OS.Windows import security_sys
# from DOM.W3C.NamedNodeMap import NamedNodeMap

import logging
log = logging.getLogger("Thug")


class Node(etree.ElementBase):
    def __init__(self, elementName):
        self._nodeTypedValue = None
        self._dataType = None
        etree.ElementBase.__init__(elementName)

    def getNodeTypedValue(self):
        try:
            if self._dataType == 'bin.base64':
                return base64.b64decode(self.text)
            elif self._dataType == 'bin.hex':
                return binascii.unhexlify(self.text)
        except:  # pylint:disable=bare-except
            pass

        return self.text

    def setNodeTypedValue(self, value):
        try:
            if self.dataType == 'bin.base64':
                self.text = base64.b64encode(value)
            elif self.dataType == 'bin.hex':
                self.text = binascii.hexlify(value)
            else:
                self.text = value
        except:  # pylint:disable=bare-except
            self.text = value

    nodeTypedValue = property(getNodeTypedValue, setNodeTypedValue)

    def getDataType(self):
        return self._dataType

    def setDataType(self, value):
        self._dataType = value

    dataType = property(getDataType, setDataType)


def loadXML(self, bstrXML):
    self.xml = w3c.parseString(bstrXML)
    # self.attributes = NamedNodeMap(self.xml._node)

    if "res://" not in bstrXML:
        return

    for p in bstrXML.split('"'):
        if p.startswith("res://"):
            log.URLClassifier.classify(p)
            log.ThugLogging.add_behavior_warn("[Microsoft XMLDOM ActiveX] Attempting to load %s" % (p, ))
            log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2017-0022", None)
            if any(sys.lower() in p.lower() for sys in security_sys):
                self.parseError._errorCode = 0

    for p in bstrXML.split("'"):
        if p.startswith("res://"):
            log.URLClassifier.classify(p)
            log.ThugLogging.add_behavior_warn("[Microsoft XMLDOM ActiveX] Attempting to load %s" % (p, ))
            log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2017-0022", None)
            if any(sys.lower() in p.lower() for sys in security_sys):
                self.parseError._errorCode = 0


def createElement(self, bstrTagName):
    log.ThugLogging.add_behavior_warn("[Microsoft XMLDOM ActiveX] Creating element %s" % (bstrTagName, ))
    return Node(bstrTagName)



# Microsoft Internet Explorer 6 WebViewFolderIcon
# CVE-2006-3730

import logging

log = logging.getLogger("Thug")


def setSlice(self, arg0, arg1, arg2, arg3):
    log.ThugLogging.add_behavior_warn('[WebViewFolderIcon ActiveX] setSlice(%s, %s, %s, %s)' % (arg0, arg1, arg2, arg3, ))
    if arg0 == 0x7ffffffe:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "WebViewFolderIcon ActiveX",
                                          "setSlice attack",
                                          cve = 'CVE-2006-3730')

        log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2006-3730", None)




import string
import time
import random
import re
import six
import logging
import hashlib
import pefile
import PyV8

from thug.Magic.Magic import Magic

log = logging.getLogger("Thug")


class _Environment(object):
    def __init__(self, strType):
        self.strType = strType

    def Item(self, item):
        log.ThugLogging.add_behavior_warn("[WScript.Shell ActiveX] Getting Environment Item: %s" % (item, ))
        return item


def Run(self, strCommand, intWindowStyle = 1, bWaitOnReturn = False):
    log.ThugLogging.add_behavior_warn("[WScript.Shell ActiveX] Executing: %s" % (strCommand, ))
    log.ThugLogging.log_exploit_event(self._window.url,
                                      "WScript.Shell ActiveX",
                                      "Run",
                                      data = {
                                                "command" : strCommand
                                             },
                                      forward = False)

    data = {
        'content' : strCommand,
    }

    log.ThugLogging.log_location(log.ThugLogging.url, data)
    log.TextClassifier.classify(log.ThugLogging.url, strCommand)

    if 'http' not in strCommand:
        return

    self._doRun(strCommand, 1)


def _doRun(self, p, stage):
    if not isinstance(p, six.string_types):
        return

    try:
        pefile.PE(data = p, fast_load = True)
        return
    except:  # pylint:disable=bare-except
        pass

    if log.ThugOpts.code_logging:
        log.ThugLogging.add_code_snippet(p, 'VBScript', 'Contained_Inside')

    log.ThugLogging.add_behavior_warn("[Wscript.Shell ActiveX] Run (Stage %d) Code:\n%s" % (stage, p))
    log.ThugLogging.log_exploit_event(self._window.url,
                                      "WScript.Shell ActiveX",
                                      "Run",
                                      data = {
                                                "stage" : stage,
                                                "code"  : p,
                                             },
                                      forward = False)

    while True:
        try:
            index = p.index('"http')
        except ValueError:
            break

        p = p[index + 1:]
        s = p.split('"')
        if len(s) < 2:
            break

        url = s[0]
        log.add_behavior_warn("[Wscript.Shell ActiveX] Run (Stage %d) Downloading from URL %s" % (stage, url))

        try:
            response = self._window._navigator.fetch(url, redirect_type = "doRun")
        except:  # pylint:disable=bare-except
            continue

        if response is None:
            continue

        if response.status_code == 404:
            continue

        md5 = hashlib.md5()
        md5.update(response.content)
	md5sum = md5.hexdigest()
        sha256 = hashlib.sha256()
        sha256.update(response.content)
	sha256sum = sha256.hexdigest()

        log.ThugLogging.add_behavior_warn("[Wscript.Shell ActiveX] Run (Stage %d) Saving file %s" % (stage, md5sum, ))
        p = '"'.join(s[1:])

        data = {
                'status'  : response.status_code,
                'content' : response.content,
                'md5'	  : md5sum,
                'sha256'  : sha256sum,
                'fsize'   : len(response.content),
                'ctype'   : response.headers.get('content-type', 'unknown'),
                'mtype'   : Magic(response.content).get_mime(),
        }

        log.ThugLogging.log_location(url, data)
        log.TextClassifier.classify(url, response.content)

        self._doRun(response.content, stage + 1)


def Environment(self, strType = None):
    log.ThugLogging.add_behavior_warn('[WScript.Shell ActiveX] Environment("%s")' % (strType, ))
    log.ThugLogging.log_exploit_event(self._window.url,
                                      "WScript.Shell ActiveX",
                                      "Environment",
                                      data = {
                                                "env" : strType
                                             },
                                      forward = False)
    return _Environment(strType)


def ExpandEnvironmentStrings(self, strWshShell):
    log.ThugLogging.add_behavior_warn('[WScript.Shell ActiveX] Expanding environment string "%s"' % (strWshShell, ))
    log.ThugLogging.log_exploit_event(self._window.url,
                                      "WScript.Shell ActiveX",
                                      "ExpandEnvironmentStrings",
                                      data = {
                                                "wshshell" : strWshShell
                                             },
                                      forward = False)

    # Substitute shell variables
    strWshShell = re.sub(
        r'%([a-zA-Z-0-9_\(\)]+)%',
        (lambda m: log.ThugOpts.Personality.getShellVariable(m.group(1))),
        strWshShell)

    # Generate random username
    strWshShell = strWshShell.replace(
        '{{username}}',
        ''.join(random.choice(string.ascii_lowercase + string.digits) for _ in range(8)))

    # Generate random computer name
    strWshShell = strWshShell.replace(
        '{{computername}}',
        ''.join(random.choice(string.ascii_uppercase + string.digits) for _ in range(8)))

    log.ThugLogging.add_behavior_warn('[WScript.Shell ActiveX] Expanded environment string to "%s"' % (strWshShell, ))
    return strWshShell


def CreateObject(self, strProgID, strPrefix = ""):
    import thug.ActiveX as ActiveX

    log.ThugLogging.add_behavior_warn("[WScript.Shell ActiveX] CreateObject (%s)" % (strProgID))
    log.ThugLogging.log_exploit_event(self._window.url,
                                      "WScript.Shell ActiveX",
                                      "CreateObject",
                                      data = {
                                          "strProgID": strProgID,
                                          "strPrefix": strPrefix
                                      },
                                      forward = False)

    return ActiveX.ActiveX._ActiveXObject(self._window, strProgID)


def Sleep(self, intTime):
    log.ThugLogging.add_behavior_warn("[WScript.Shell ActiveX] Sleep(%s)" % (intTime))
    time.sleep(intTime * 0.001)


def Quit(self, code):
    log.ThugLogging.add_behavior_warn("[WScript.Shell ActiveX] Quit(%s)" % code)
    PyV8.JSEngine.terminateAllThreads()


def Echo(self, text):
    log.ThugLogging.add_behavior_warn("[WScript.Shell ActiveX] Echo(%s)" % (text))


def valueOf(self):
    return "Windows Script Host"


def toString(self):
    return "Windows Script Host"


def SpecialFolders(self, strFolderName):
    log.ThugLogging.add_behavior_warn('[WScript.Shell ActiveX] Received call to SpecialFolders property "%s"' % (strFolderName, ))
    folderPath = log.ThugOpts.Personality.getSpecialFolder(strFolderName)
    if folderPath:
        folderPath = ExpandEnvironmentStrings(self, folderPath)
    return "{}".format(folderPath)


def CreateShortcut(self, strPathname):
    log.ThugLogging.add_behavior_warn('[WScript.Shell ActiveX] CreateShortcut "%s"' % (strPathname, ))
    obj = CreateObject(self, "wscript.shortcut")
    obj.FullName = strPathname
    return obj


def RegRead(self, registry):
    reg_map = {
        'hklm\software\microsoft\windows\currentversion\programfilesdir' : 'ProgramFiles',
    }

    win32_registry = getattr(log, 'win32_registry', None)
    if win32_registry and registry.lower() in win32_registry:
        value = win32_registry[registry.lower()]
        log.ThugLogging.add_behavior_warn('[WScript.Shell ActiveX] RegRead("{}") = "{}"'.format(registry, value))
        return value

    if registry.lower() in reg_map:
        value = log.ThugOpts.Personality.getShellVariable(reg_map[registry.lower()])
        log.ThugLogging.add_behavior_warn('[WScript.Shell ActiveX] RegRead("{}") = "{}"'.format(registry, value))
        return value

    log.ThugLogging.add_behavior_warn('[WScript.Shell ActiveX] RegRead("{}") = {}'.format(registry, 'NOT FOUND'))
    return ''


def RegWrite(self, registry, value, strType = "REG_SZ"):
    log.ThugLogging.add_behavior_warn('[WScript.Shell ActiveX] RegWrite("{}", "{}", "{}")'.format(registry, value, strType))

    win32_registry = getattr(log, 'win32_registry', None)
    if win32_registry is None:
        log.win32_registry = dict()

    log.win32_registry[registry.lower()] = value



#!/usr/bin/env python

from thug.DOM.W3C.DOMException import DOMException
from .HTMLCollection import HTMLCollection


# Introduced in DOM Level 2
class HTMLOptionsCollection(HTMLCollection):
    def __init__(self, doc, nodes):
        HTMLCollection.__init__(self, doc, nodes)

    def getLength(self):
        return len(self.nodes)

    def setLength(self):
        raise DOMException(DOMException.NOT_SUPPORTED_ERR)

    length = property(getLength, setLength)



#!/usr/bin/env python
#
# Personality.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA

import sys
import os
import json
import logging

log = logging.getLogger("Thug")


class Personality(dict):
    def __init__(self):
        personalities = log.personalities_path

        if personalities is None:
            log.warning("[CRITICAL] Thug personalities not found! Exiting")
            sys.exit(0)

        for root, _dir, files in os.walk(personalities):  # pylint:disable=unused-variable
            for f in files:
                if not f.endswith('.json'):
                    continue

                name = f.split(".json")[0]
                with open(os.path.join(root, f)) as personality:
                    self[name] = json.load(personality)

                # Shell variables are case insensitive, will process them
                # to make sure all keys in the dict are lowercase.
                for k, v in self[name].pop('shellVariables', dict()).iteritems():
                    if 'shellVariables' not in self[name]:
                        self[name]['shellVariables'] = dict()

                    self[name]['shellVariables'][k.lower()] = v

                # Special folder names are case insensitive, will process them
                # to make sure all keys in the dict are lowercase.
                for k, v in self[name].pop('specialFolders', dict()).iteritems():
                    if 'specialFolders' not in self[name]:
                        self[name]['specialFolders'] = dict()

                    self[name]['specialFolders'][k.lower()] = v

    @property
    def userAgent(self):
        return self[log.ThugOpts.useragent]['userAgent']

    @property
    def javaUserAgent(self):
        return self[log.ThugOpts.useragent]['javaUserAgent']

    @property
    def browserVersion(self):
        return self[log.ThugOpts.useragent]['version']

    @property
    def platform(self):
        return self[log.ThugOpts.useragent]['platform']

    @property
    def browserMajorVersion(self):
        return int(self.browserVersion.split('.')[0])

    @property
    def cc_on(self):
        return self[log.ThugOpts.useragent]['cc_on']

    def isIE(self):
        return self[log.ThugOpts.useragent]['browserTag'].startswith('ie')

    def isEdge(self):
        return self[log.ThugOpts.useragent]['browserTag'].startswith('edge')

    def isWindows(self):
        return log.ThugOpts.useragent.startswith('win')

    def isFirefox(self):
        return self[log.ThugOpts.useragent]['browserTag'].startswith('firefox')

    def isChrome(self):
        return self[log.ThugOpts.useragent]['browserTag'].startswith('chrome')

    def isSafari(self):
        return self[log.ThugOpts.useragent]['browserTag'].startswith('safari')

    def isOpera(self):
        return self[log.ThugOpts.useragent]['browserTag'].startswith('opera')

    def ScriptEngineMajorVersion(self):
        return self[log.ThugOpts.useragent]['ScriptEngineMajorVersion']

    def ScriptEngineMinorVersion(self):
        return self[log.ThugOpts.useragent]['ScriptEngineMinorVersion']

    def ScriptEngineBuildVersion(self):
        return self[log.ThugOpts.useragent]['ScriptEngineBuildVersion']

    def getShellVariable(self, variableName):
        return self[log.ThugOpts.useragent].get('shellVariables', dict()).get(variableName.strip("%").lower(), '')

    def getSpecialFolder(self, folderName):
        return self[log.ThugOpts.useragent].get('specialFolders', dict()).get(folderName.lower(), '')



# iMesh<= 7.1.0.x IMWebControl Class
# CVE-2007-6493, CVE-2007-6492

import logging

log = logging.getLogger("Thug")


def ProcessRequestEx(self, arg):
    if len(arg) == 0:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "iMesh IMWebControl ActiveX",
                                          "NULL value in ProcessRequestEx",
                                          cve = 'CVE-2007-6492')

        log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2007-6492", None)


def SetHandler(self, arg):
    if str([arg]) == '218959117':
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "iMesh IMWebControl ActiveX",
                                          "Overflow in SetHandler",
                                          cve = 'CVE-2007-6493')

        log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2007-6493", None)



# Universal HTTP File Upload (UUploaderSverD.dll - v6.0.0.35)
# CVE-NOMATCH

import logging

log = logging.getLogger("Thug")


def RemoveFileOrDir(self, arg0, arg1):
    log.ThugLogging.add_behavior_warn('[Universal HTTP File Upload ActiveX] Deleting %s' % (arg0, ))
    log.ThugLogging.log_exploit_event(self._window.url,
                                      "Universal HTTP File Upload ActiveX",
                                      "Deleting",
                                      data = {
                                                "filename": arg0
                                             },
                                      forward = False)



# Microsoft Windows Media Encoder WMEX.DLL ActiveX BufferOverflow vulnerability
# CVE-2008-3008

import logging

log = logging.getLogger("Thug")


def GetDetailsString(self, arg0, arg1):
    if len(arg0) > 1023:
        log.ThugLogging.add_behavior_warn('[Microsoft Windows Media Encoder WMEX.DLL ActiveX] GetDetailsString Method Buffer Overflow',
                                          'CVE-2008-3008')
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "Microsoft Windows Media Encoder WMEX.DLL ActiveX",
                                          "GetDetailsString Method Buffer Overflow",
                                          cve = "CVE-2008-3008")

        log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2008-3008", None)



#!/usr/bin/env python

from .HTMLElement import HTMLElement
from .attr_property import attr_property
from .compatibility import thug_long


class HTMLButtonElement(HTMLElement):
    def __init__(self, doc, tag):
        HTMLElement.__init__(self, doc, tag)

    @property
    def form(self):
        pass

    accessKey       = attr_property("accesskey")
    disabled        = attr_property("disabled", bool)
    name            = attr_property("name")
    tabIndex        = attr_property("tabindex", thug_long)
    type            = attr_property("type")
    value           = attr_property("value")

    def onclick(self):
        pass



# Microsoft Rich Textbox Control 6.0 (SP6)
# CVE-NOMATCH

import logging

log = logging.getLogger("Thug")


def SaveFile(self, path, arg):
    log.ThugLogging.add_behavior_warn("[Microsoft Rich Textbox Control ActiveX] Writing to file %s" % (str(path), ))
    log.ThugLogging.add_behavior_warn("[Microsoft Rich Textbox Control ActiveX] Content: \n%s" % (str(self.Text), ))
    log.ThugLogging.log_exploit_event(self._window.url,
                                      "Microsoft Rich Textbox Control ActiveX",
                                      "Writing file",
                                      data = {
                                                "file"   : str(path),
                                                "content": str(self.Text)
                                             },
                                      forward = False)



#!/usr/bin/env python

from .DOMException import DOMException
from .Node import Node
from .CharacterData import CharacterData


class Text(CharacterData):
    def __repr__(self):
        return "<Text '%s' at 0x%08X>" % (self.tag, id(self))

    def splitText(self, offset):
        raise DOMException(DOMException.NO_MODIFICATION_ALLOWED_ERR)

    def getNodeValue(self):
        return str(self.data)

    def setNodeValue(self, data):
        self.data = data

    nodeValue = property(getNodeValue, setNodeValue)

    @property
    def nodeName(self):
        return "#text"

    @property
    def nodeType(self):
        return Node.TEXT_NODE

    def replaceData(self, offset, count, arg):
        s = self.data[:offset] + arg + self.data[offset + count:]
        self.data = s
        # raise DOMException(DOMException.NO_MODIFICATION_ALLOWED_ERR)



#!/usr/bin/env python
#
# ClipboardData.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA

from .JSClass import JSClass


class ClipboardData(JSClass):
    def __init__(self):
        self._data = dict()

    def getData(self, dataFormat):
        if dataFormat in self._data:
            return self._data[dataFormat]

        return None

    def setData(self, dataFormat, data):
        if dataFormat not in ('Text', 'URL'):
            return False

        self._data[dataFormat] = data
        return True

    def clearData(self, dataFormat = None):
        if dataFormat is None:
            self._data.clear()
            return

        if dataFormat in self._data:
            del self._data[dataFormat]



#!/usr/bin/env python
#
# Plugins.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA


class Plugins(list):
    def __init__(self):
        list.__init__(self)

    @property
    def length(self):
        return len(self)

    def __getattr__(self, key):
        return self.namedItem(key)

    def __getitem__(self, key):
        try:
            key = int(key)
            return self.item(key)
        except ValueError:
            return self.namedItem(key)

    def item(self, index):
        if index >= self.length:
            return None

        return list.__getitem__(self, index)

    def namedItem(self, name):
        index = 0

        while index < self.length:
            p = self.item(index)
            if p['name'] == name:
                return p

            index += 1

        return None

    def refresh(self, reloadDocuments = False):
        pass



# Microsoft MDAC RDS.Dataspace ActiveX
# CVE-2006-0003

import logging

log = logging.getLogger("Thug")


def CreateObject(self, _object, param = ''):
    import thug.ActiveX as ActiveX

    log.ThugLogging.add_behavior_warn("[Microsoft MDAC RDS.Dataspace ActiveX] CreateObject (%s)" % (_object))
    log.ThugLogging.log_exploit_event(self._window.url,
                                      "Microsoft MDAC RDS.Dataspace ActiveX",
                                      "CreateObject",
                                      cve = "CVE-2006-0003",
                                      forward = False,
                                      data = {
                                                "object": _object
                                             }
                                     )

    log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2006-0003", None)
    return ActiveX.ActiveX._ActiveXObject(self._window, _object)



#!/usr/bin/env python

import logging
from .UIEvent import UIEvent

log = logging.getLogger("Thug")


# Introduced in DOM Level 2
class MouseEvent(UIEvent):
    MouseEventTypes = ('click', 'mousedown', 'mouseup', 'mouseover', 'mousemove', 'mouseout')

    def __init__(self, typeArg, target):
        UIEvent.__init__(self, typeArg, target)
        canBubbleArg  = typeArg in ('click', 'mousedown', 'mouseup', 'mouseover', 'mousemove', 'mouseout')
        cancelableArg = typeArg in ('click', 'mousedown', 'mouseup', 'mouseover', 'mouseout')
        self.initMouseEvent(typeArg          = typeArg,
                            canBubbleArg     = canBubbleArg,
                            cancelableArg    = cancelableArg,
                            relatedTargetArg = target)

    @property
    def altKey(self):
        return self._altKey

    @property
    def button(self):
        return self._button

    @property
    def clientX(self):
        return self._clientX

    @property
    def clientY(self):
        return self._clientY

    @property
    def ctrlKey(self):
        return self._ctrlKey

    @property
    def metaKey(self):
        return self._metaKey

    @property
    def relatedTarget(self):
        return self._relatedTarget

    @property
    def screenX(self):
        return self._screenX

    @property
    def screenY(self):
        return self._screenY

    @property
    def shiftKey(self):
        return self._shiftKey

    @property
    def detail(self):
        return self._detail

    def initMouseEvent(self,
                       typeArg,
                       canBubbleArg,
                       cancelableArg,
                       viewArg            = None,
                       detailArg          = 1,
                       screenXArg         = 0,
                       screenYArg         = 0,
                       clientXArg         = 0,
                       clientYArg         = 0,
                       ctrlKeyArg         = False,
                       altKeyArg          = False,
                       shiftKeyArg        = False,
                       metaKeyArg         = False,
                       buttonArg          = 1,
                       relatedTargetArg   = None):
        log.debug('initMouseEvent(%s, %s, %s, %s, %s)', typeArg,
                                                        canBubbleArg,
                                                        cancelableArg,
                                                        viewArg,
                                                        detailArg)

        self._screenX       = screenXArg
        self._screenY       = screenYArg
        self._clientX       = clientXArg
        self._clientY       = clientYArg
        self._ctrlKey       = ctrlKeyArg
        self._altKey        = altKeyArg
        self._shiftKey      = shiftKeyArg
        self._metaKey       = metaKeyArg
        self._button        = buttonArg
        self._relatedTarget = relatedTargetArg
        self.initUIEvent(typeArg, canBubbleArg, cancelableArg, viewArg, detailArg)



import six
import collections 

from thug.DOM.JSClass import JSClass


class Fields(JSClass):
    def __init__(self, items = None):
        self.items = collections.OrderedDict() if items is None else items

    @property
    def count(self):
        return len(self.items)

    def Item(self, key):
        if key in six.string_types:
            item = getattr(self.items, key, None)
            if item:
                return item

        try:
            index = int(key)
        except ValueError:
            raise

        try:
            item = self.items[index]
        except KeyError:
            raise

        return item



#!/usr/bin/env python

from thug.DOM.JSClass import JSClass


class NodeList(JSClass):
    def __init__(self, doc, nodes):
        self.doc = doc
        self.nodes = nodes

    def __len__(self):
        return self.length

    def __getitem__(self, key):
        return self.item(int(key))

    def item(self, index):
        from thug.DOM.W3C.DOMImplementation import DOMImplementation
        return DOMImplementation.createHTMLElement(self.doc, self.nodes[index]) if 0 <= index and index < len(self.nodes) else None

    @property
    def length(self):
        return len(self.nodes)




import logging

log = logging.getLogger("Thug")


def SetWksPictureInterface(self, val):
    self.__dict__['WksPictureInterface'] = val

    log.ThugLogging.log_exploit_event(self._window.url,
                                      "MicrosoftWorks7 ActiveX",
                                      "Overflow in WksPictureInterface property")




import logging

log = logging.getLogger("Thug")


def ShockwaveVersion(self, arg):
    if len(arg) >= 768 * 768:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "Shockwave",
                                          "ShockwaveVersion Stack Overflow")



# Chinagames iGame CGAgent ActiveX Control Buffer Overflow
# CVE-2009-1800

import logging

log = logging.getLogger("Thug")


def CreateChinagames(self, arg0):
    if len(arg0) > 428:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "CGAgent ActiveX",
                                          "CreateChinagames Method Buffer Overflow",
                                          cve = 'CVE-2009-1800')

        log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2009-1800", None)



#!/usr/bin/env python
#
# ExploitGraph.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA

import json
import logging

NETWORKX_MODULE = True

try:
    import networkx
    from networkx.readwrite import json_graph
except ImportError:
    NETWORKX_MODULE = False

log = logging.getLogger("Thug")


class ExploitGraph(object):
    def __init__(self, url):
        self.enabled = NETWORKX_MODULE
        if not self.enabled:
            return

        self.G = networkx.MultiDiGraph(url = url, name = "thug-exploit-graph")

    def add_connection(self, source, destination, method):
        if not self.enabled:
            return

        self.G.add_node(source)
        self.G.add_node(destination)
        self.G.add_edge(source, destination, method = method)

    def draw(self):
        if not self.enabled:
            return None

        G = networkx.convert_node_labels_to_integers(self.G, label_attribute = 'url')
        d = json_graph.node_link_data(G)

        return json.dumps(d)



#!/usr/bin/env python

import logging

from .HTMLElement import HTMLElement
from .attr_property import attr_property
from .compatibility import thug_long

log = logging.getLogger("Thug")


class HTMLObjectElement(HTMLElement):
    def __init__(self, doc, tag):
        HTMLElement.__init__(self, doc, tag)
        self._window = self.doc.window

    def __getattr__(self, name):
        for (key, value) in self.tag.attrs.iteritems():
            if key != 'id':
                continue

            _obj = getattr(self.doc.window, value)
            if _obj:
                return getattr(_obj, name)

        if name in self.__dict__:
            return self.__dict__[name]

        log.info("HTMLObjectElement attribute not found: %s", (name, ))

    # PLEASE REVIEW ME!
    def __setattr__(self, name, value):
        if name == 'classid':
            self.setAttribute(name, value)
            return

        self.__dict__[name] = value

        if 'funcattrs' not in self.__dict__:
            return

        if name in self.__dict__['funcattrs']:
            self.__dict__['funcattrs'][name](value)

    @property
    def form(self):
        pass

    code            = attr_property("code")
    align           = attr_property("align")
    archive         = attr_property("archive")
    border          = attr_property("border")
    classid         = attr_property("classid")
    codeBase        = attr_property("codebase")
    codeType        = attr_property("codetype")
    data            = attr_property("data")
    declare         = attr_property("declare", bool)
    height          = attr_property("height")
    hspace          = attr_property("hspace", thug_long)
    name            = attr_property("name")
    standBy         = attr_property("standby")
    tabIndex        = attr_property("tabindex", thug_long, default = 0)
    type            = attr_property("type")
    useMap          = attr_property("usemap")
    vspace          = attr_property("vspace", thug_long)
    width           = attr_property("width")

    # Introduced in DOM Level 2
    @property
    def contentDocument(self):
        return self.doc if self.doc else None

    def setAttribute(self, name, value):
        # ActiveX registration
        if name == 'classid':
            from thug.ActiveX.ActiveX import register_object

            try:
                register_object(self, value)
            except:  # pylint:disable=bare-except
                return

        self.tag[name] = value

    @property
    def object(self):
        return self

    def definition(self, value):
        pass



#!/usr/bin/env python
#
# CLSID.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA

from .modules import AcroPDF
from .modules import AdodbRecordset
from .modules import AdodbStream
from .modules import AnswerWorks
from .modules import AolAmpX
from .modules import AolICQ
from .modules import AOLAttack
from .modules import BaiduBar
from .modules import BitDefender
from .modules import CABrightStor
from .modules import CGAgent
from .modules import Comodo
from .modules import ConnectAndEnterRoom
from .modules import CreativeSoftAttack
from .modules import DirectShow
from .modules import DivX
from .modules import DLinkMPEG
from .modules import Domino
from .modules import DPClient
from .modules import DVRHOSTWeb
from .modules import EnjoySAP
from .modules import FacebookPhotoUploader
from .modules import FileUploader
from .modules import GatewayWeblaunch
from .modules import GLIEDown2
from .modules import Gogago
from .modules import GomWeb
from .modules import HPInfo
from .modules import ICQToolbar
from .modules import IMWebControl
from .modules import InternetCleverSuite
from .modules import JavaDeploymentToolkit
from .modules import JetAudioDownloadFromMusicStore
from .modules import Kingsoft
from .modules import MacrovisionFlexNet
from .modules import MicrosoftWorks7Attack
from .modules import MicrosoftXMLDOM
from .modules import MicrosoftXMLHTTP
from .modules import Move
from .modules import MSRICHTXT
from .modules import MSVFP
from .modules import MSXML2DOMDocument
from .modules import MyspaceUploader
from .modules import NamoInstaller
from .modules import NCTAudioFile2
from .modules import NeoTracePro
from .modules import NessusScanCtrl
from .modules import OfficeOCX
from .modules import OurgameGLWorld
from .modules import PPlayer
from .modules import PTZCamPanel
from .modules import QuantumStreaming
from .modules import QvodCtrl
from .modules import RDSDataSpace
from .modules import RealPlayer
from .modules import RediffBolDownloaderAttack
from .modules import RegistryPro
from .modules import RisingScanner
from .modules import RtspVaPgCtrl
from .modules import ScriptingFileSystemObject
from .modules import ShellApplication
from .modules import Shockwave
from .modules import ShockwaveFlash9
from .modules import ShockwaveFlash10
from .modules import ShockwaveFlash11
from .modules import ShockwaveFlash12
from .modules import SilverLight
from .modules import SinaDLoader
from .modules import SnapshotViewer
from .modules import SonicWallNetExtenderAddRouteEntry
from .modules import Spreadsheet
from .modules import SSReaderPdg2
from .modules import StormConfig
from .modules import StormMps
from .modules import SymantecAppStream
from .modules import SymantecBackupExec
from .modules import StreamAudioChainCast
from .modules import Toshiba
from .modules import UniversalUpload
from .modules import UUSeeUpdate
from .modules import VisualStudioDTE80
from .modules import VLC
from .modules import VsaIDEDTE
from .modules import VsmIDEDTE
from .modules import WebViewFolderIcon
from .modules import WindowsMediaPlayer
from .modules import WinZip
from .modules import WMEncProfileManager
from .modules import WMP
from .modules import WScriptShell
from .modules import WScriptShortcut
from .modules import WScriptNetwork
from .modules import XMLDOMParseError
from .modules import XUpload
from .modules import YahooJukebox
from .modules import YahooMessengerCyft
from .modules import YahooMessengerYVerInfo
from .modules import YahooMessengerYwcvwr
from .modules import ZenturiProgramCheckerAttack

import logging
log = logging.getLogger("Thug")


CLSID = [
        # AcroPDF.PDF
        {
            'id'        : ( 'CA8A9780-280D-11CF-A24D-444553540000', ),
            'name'      : ( 'acropdf.pdf', 'pdf.pdfctrl', ),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'GetVersions'   : AcroPDF.GetVersions,
                          }
        },

        # Adodb.Recordset
        {
            'id'        : (),
            'name'      : ( 'adodb.recordset', ),
            'attrs'     : {
                            'Fields'        : AdodbRecordset.Fields(),
                          },
            'funcattrs' : {},
            'methods'   : {}
        },

        # Adodb.Stream
        {
            'id'        : (),
            'name'      : ( 'adodb.stream', ),
            'attrs'     : {
                            'Charset'       : 'Unicode',
                            'type'          : 1,
                            'Type'          : 1,
                            'Mode'          : 3,
                            'Position'      : 0,
                            'position'      : 0,
                            'Size'          : 0,
                            '_files'        : dict(),
                            '_current'      : None},
            'funcattrs' : {
                            'position'      : AdodbStream.setPosition,
            },
            'methods'   : {
                            'Open'          : AdodbStream.open,
                            'Read'          : AdodbStream.Read,
                            'Write'         : AdodbStream.Write,
                            'SaveToFile'    : AdodbStream.SaveToFile,
                            'SaveTofile'    : AdodbStream.SaveToFile,
                            'LoadFromFile'  : AdodbStream.LoadFromFile,
                            'ReadText'      : AdodbStream.ReadText,
                            'WriteText'     : AdodbStream.WriteText,
                            'Close'         : AdodbStream.Close,
                            'setPosition'   : AdodbStream.setPosition,
                         }
        },

        # AnswerWorks
        {
            'id'        : ( 'C1908682-7B2C-4AB0-B98E-183649A0BF84', ),
            'name'      : ( 'awapi4.answerworks.1'),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'GetHistory'    : AnswerWorks.GetHistory,
                            'GetSeedQuery'  : AnswerWorks.GetSeedQuery,
                            'SetSeedQuery'  : AnswerWorks.SetSeedQuery,
                          }
        },

        # AolAmpX
        {
            'id'        : ( 'B49C4597-8721-4789-9250-315DFBD9F525',
                            'FA3662C3-B8E8-11D6-A667-0010B556D978',
                            'FE0BD779-44EE-4A4B-AA2E-743C63F2E5E6' ),
            'name'      : (),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'AppendFileToPlayList'  : AolAmpX.AppendFileToPlayList,
                            'ConvertFile'           : AolAmpX.ConvertFile,
                          }
        },

        # AolICQ
        {
            'id'        : (),
            'name'      : ( 'icqphone.sipxphonemanager.1', ),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'DownloadAgent'         : AolICQ.DownloadAgent,
                          }
        },

        # AOLAttack
        {
            'id'        : ( '189504B8-50D1-4AA8-B4D6-95C8F58A6414', ),
            'name'      : ( 'sb.superbuddy', 'sb.superbuddy.1', ),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'LinkSBIcons'           : AOLAttack.LinkSBIcons,
                          }
        },

        # BaiduBar
        {
            'id'        : ( 'A7F05EE4-0426-454F-8013-C41E3596E9E9', ),
            'name'      : ( 'baidubar.tool', ),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'DloadDS'               : BaiduBar.DloadDS,
                          }
        },

        # BitDefender
        {
            'id'        : ( '5D86DDB5-BDF9-441B-9E9E-D4730F4EE499', ),
            'name'      : ( ),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'initx'                 : BitDefender.initx,
                          }
        },

        # CABrightStor
        {
            'id'        : ( 'BF6EFFF3-4558-4C4C-ADAF-A87891C5F3A3', ),
            'name'      : ( 'listctrl.listctrlctrl.1', ),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'AddColumn'              : CABrightStor.AddColumn,
                          }
        },

        # CGAgent
        {
            'id'        : ( '75108B29-202F-493C-86C5-1C182A485C4C', ),
            'name'      : ( 'cgagent.agent.1', ),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'CreateChinagames'       : CGAgent.CreateChinagames,
                          }
        },

        # Comodo
        {
            'id'        : ( '309F674D-E4D3-46BD-B9E2-ED7DFD7FD176', ),
            'name'      : ( ),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'ExecuteStr'             : Comodo.ExecuteStr,
                          }
        },

        # ConnectAndEnterRoom
        {
            'id'        : ( 'AE93C5DF-A990-11D1-AEBD-5254ABDD2B69', ),
            'name'      : ( 'glchat.glchatctrl.1', ),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'ConnectAndEnterRoom'    : ConnectAndEnterRoom.ConnectAndEnterRoom,
                          }
        },

        # CreativeSoftAttack
        {
            'id'        : ( '0A5FD7C5-A45C-49FC-ADB5-9952547D5715', ),
            'name'      : (),
            'attrs'     : {
                            'cachefolder'           : '',
                          },
            'funcattrs' : {
                            'cachefolder'           : CreativeSoftAttack.Setcachefolder,
                          },
            'methods'   : {
                            'Setcachefolder'        : CreativeSoftAttack.Setcachefolder,
                          }
        },

        # DirectShow
        {
            'id'        : ( '0955AC62-BF2E-4CBA-A2B9-A63F772D46CF', ),
            'name'      : (),
            'attrs'     : {
                            'data'                  : '',
                            'width'                 : '1',
                            'height'                : '1',
                          },
            'funcattrs' : {
                            'data'                  : DirectShow.Setdata,
                          },
            'methods'   : {
                            'Setdata'               : DirectShow.Setdata,
                          }
        },

        # DivX
        {
            'id'        : ( 'D050D736-2D21-4723-AD58-5B541FFB6C11', ),
            'name'      : (),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'SetPassword'           : DivX.SetPassword,
                          }
        },

        # DLinkMPEG
        {
            'id'        : ( 'A93B47FD-9BF6-4DA8-97FC-9270B9D64A6C', ),
            'name'      : (),
            'attrs'     : {
                            'Url'                   : ''
                          },
            'funcattrs' : {
                            'Url'                   : DLinkMPEG.SetUrl,
                          },
            'methods'   : {
                            'SetUrl'                : DLinkMPEG.SetUrl,
                          }
        },

        # Domino
        {
            'id'        : ( 'E008A543-CEFB-4559-912F-C27C2B89F13B',
                            '3BFFE033-BF43-11D5-A271-00A024A51325',
                            '983A9C21-8207-4B58-BBB8-0EBC3D7C5505',
                          ),
            'name'      : (),
            'attrs'     : {
                            'General_ServerName'        : '',
                            'General_JunctionName'      : '',
                            'Mail_MailDbPath'           : '',
                          },
            'funcattrs' : {
                            'General_ServerName'        : Domino.SetGeneral_ServerName,
                            'General_JunctionName'      : Domino.SetGeneral_JunctionName,
                            'Mail_MailDbPath'           : Domino.SetMail_MailDbPath
                          },
            'methods'   : {
                            'SetGeneral_ServerName'     : Domino.SetGeneral_ServerName,
                            'SetGeneral_JunctionName'   : Domino.SetGeneral_JunctionName,
                            'SetMail_MailDbPath'        : Domino.SetMail_MailDbPath,
                            'InstallBrowserHelperDll'   : Domino.InstallBrowserHelperDll,
                          }
        },

        # DPClient
        {
            'id'        : ( 'EEDD6FF9-13DE-496B-9A1C-D78B3215E266', ),
            'name'      : ( 'dpclient.vod.1', ),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'DownURL2'          : DPClient.DownURL2,
                          }
        },

        # DVRHOSTWeb
        {
            'id'        : ( 'D64CF6D4-45DF-4D8F-9F14-E65FADF2777C', ),
            'name'      : ( 'pdvratl.pdvrocx.1', ),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'TimeSpanFormat'    : DVRHOSTWeb.TimeSpanFormat,
                          }
        },

        # EnjoySAP
        {
            'id'        : ( '2137278D-EF5C-11D3-96CE-0004AC965257', ),
            'name'      : ( 'rfcguisink.rfcguisink.1', 'kweditcontrol.kwedit.1', ),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'LaunchGui'         : EnjoySAP.LaunchGui,
                            'PrepareToPostHTML' : EnjoySAP.PrepareToPostHTML,
                            'Comp_Download'     : EnjoySAP.Comp_Download
                          }
        },

        # FacebookPhotoUploader
        {
            'id'        : ( '6E5E167B-1566-4316-B27F-0DDAB3484CF7',
                            '5C6698D9-7BE4-4122-8EC5-291D84DBD4A0',
                            'BA162249-F2C5-4851-8ADC-FC58CB424243', ),
            'name'      : ( 'thefacebook.facebookphotouploader4.4.1', ),
            'attrs'     : {
                            'ExtractIptc'       : '',
                            'ExtractExif'       : '',
                          },
            'funcattrs' : {
                            'ExtractIptc'       : FacebookPhotoUploader.SetExtractIptc,
                            'ExtractExif'       : FacebookPhotoUploader.SetExtractExif,
                          },
            'methods'   : {
                            'SetExtractIptc'    : FacebookPhotoUploader.SetExtractIptc,
                            'SetExtractExif'    : FacebookPhotoUploader.SetExtractExif,
                          }
        },

        # FileUploader
        {
            'id'        : ( 'C36112BF-2FA3-4694-8603-3B510EA3B465', ),
            'name'      : ( 'fileuploader.fuploadctl.1', ),
            'attrs'     : {
                            'HandwriterFilename'    : ''
                          },
            'funcattrs' : {
                            'HandwriterFilename'    :  FileUploader.SetHandwriterFilename,
                          },
            'methods'   : {
                            'SetHandwriterFilename' :  FileUploader.SetHandwriterFilename,
                          }
        },

        # Flash
        {
            'id'        : ( 'D27CDB6E-AE6D-11CF-96B8-444553540000', ),
            'name'      : (),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {},
        },

        # GatewayWeblaunch
        {
            'id'        : ( '93CEA8A4-6059-4E0B-ADDD-73848153DD5E',
                            '97BB6657-DC7F-4489-9067-51FAB9D8857E'),
            'name'      : (),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'DoWebLaunch'           : GatewayWeblaunch.DoWebLaunch,
                          }
        },

        # GLIEDown2
        {
            'id'        : ( 'F917534D-535B-416B-8E8F-0C04756C31A8', ),
            'name'      : ( 'gliedown.iedown.1', ),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'IEStartNative'         : GLIEDown2.IEStartNative,
                          }
        },

        # Gogago
        {
            'id'        : ( '7966A32A-5783-4F0B-824C-09077C023080', ),
            'name'      : (),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'Download'              : Gogago.Download,
                          }
        },

        # GomWeb
        {
            'id'        : ( 'DC07C721-79E0-4BD4-A89F-C90871946A31', ),
            'name'      : ( 'gomwebctrl.gommanager.1', ),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'OpenURL'              : GomWeb.OpenURL,
                          }
        },

        # HPInfo
        {
            'id'        : ( '62DDEB79-15B2-41E3-8834-D3B80493887A',
                            '7CB9D4F5-C492-42A4-93B1-3F7D6946470D', ),
            'name'      : ( 'hpinfodll.hpinfo.1', 'hprulesengine.contentcollection.1', ),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'LaunchApp'             : HPInfo.LaunchApp,
                            'SetRegValue'           : HPInfo.SetRegValue,
                            'GetRegValue'           : HPInfo.GetRegValue,
                            'EvaluateRules'         : HPInfo.EvaluateRules,
                            'SaveToFile'            : HPInfo.SaveToFile,
                            'ProcessRegistryData'   : HPInfo.ProcessRegistryData,
                          }
        },

        # Icona SpA C6 Messenger
        {
            'id'        : ( 'C1B7E532-3ECB-4E9E-BB3A-2951FFE67C61', ),
            'name'      : (),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {}
        },

        # ICQToolbar
        {
            'id'        : ( '855F3B16-6D32-4FE6-8A56-BBB695989046', ),
            'name'      : (),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'GetPropertyById'       : ICQToolbar.GetPropertyById,
                          }
        },

        # IMWebControl
        {
            'id'        : ( '7C3B01BC-53A5-48A0-A43B-0C67731134B9', ),
            'name'      : ( 'imweb.imwebcontrol.1', ),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'ProcessRequestEx'      : IMWebControl.ProcessRequestEx,
                            'SetHandler'            : IMWebControl.SetHandler,
                          }
        },

        # InternetCleverSuite
        # FIXME DOWNLOAD
        {
            'id'        : ( 'E8F92847-7C21-452B-91A5-49D93AA18F30', ),
            'name'      : (),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'GetToFile'             : InternetCleverSuite.GetToFile,
                          }
        },

        # JavaDeploymentToolkit
        {
            'id'        : ( 'CAFEEFAC-DEC7-0000-0000-ABCDEFFEDCBA',
                            'CAFEEFAC-DEC7-0000-0001-ABCDEFFEDCBA',
                            '8AD9C840-044E-11D1-B3E9-00805F499D93', ),
            'name'      : (),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'launch'                : JavaDeploymentToolkit.launch,
                            'launchApp'             : JavaDeploymentToolkit.launchApp,
                          },
        },

        # JavaPlugin
        {
            'id'        : (),
            'name'      : ( 'javaplugin', ),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {}
        },

        # JavaWebStart.isInstalled
        {
            'id'        : (),
            'name'      : ( 'javawebstart.isinstalled', ),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {}
        },

        # JetAudioDownloadFromMusicStore
        # FIXME DOWNLOAD
        {
            'id'        : ( '8D1636FD-CA49-4B4E-90E4-0A20E03A15E8', ),
            'name'      : (),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'DownloadFromMusicStore' :  JetAudioDownloadFromMusicStore.DownloadFromMusicStore,
                          }
        },

        # Kingsoft
        {
            'id'        : ( 'D82303B7-A754-4DCB-8AFC-8CF99435AACE', ),
            'name'      : (),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'SetUninstallName'      :  Kingsoft.SetUninstallName,
                          }
        },

        # MacrovisionFlexNet
        # FIXME DOWNLOAD
        {
            'id'        : ( 'FCED4482-7CCB-4E6F-86C9-DCB22B52843C',
                            '85A4A99C-8C3D-499E-A386-E0743DFF8FB7',
                            'E9880553-B8A7-4960-A668-95C68BED571E',),
            'name'      : (),
            'attrs'     : {
                            'ScheduleInterval'      : 0},
            'funcattrs' : {},
            'methods'   : {
                            'Initialize'            : MacrovisionFlexNet.Initialize,
                            'CreateJob'             : MacrovisionFlexNet.CreateJob,
                            'DownloadAndExecute'    : MacrovisionFlexNet.DownloadAndExecute,
                            'DownloadAndInstall'    : MacrovisionFlexNet.DownloadAndInstall,
                            'AddFileEx'             : MacrovisionFlexNet.AddFileEx,
                            'AddFile'               : MacrovisionFlexNet.AddFile,
                            'SetPriority'           : MacrovisionFlexNet.SetPriority,
                            'SetNotifyFlags'        : MacrovisionFlexNet.SetNotifyFlags,
                            'RunScheduledJobs'      : MacrovisionFlexNet.RunScheduledJobs,
                          }
        },

        # MicrosoftWorks7Attack
        {
            'id'        : ( '00E1DB59-6EFD-4CE7-8C0A-2DA3BCAAD9C6',  ),
            'name'      : (),
            'attrs'     : {
                            'WksPictureInterface'       : 0,
                          },
            'funcattrs' : {
                            'WksPictureInterface'       : MicrosoftWorks7Attack.SetWksPictureInterface,
                          },
            'methods'   : {
                            'SetWksPictureInterface'    : MicrosoftWorks7Attack.SetWksPictureInterface,
                          }
        },

        # MicrosoftXMLDOM
        {
            'id'        : (),
            'name'      : ( 'microsoft.xmldom', 'msxml2.domdocument.3.0', ),
            'attrs'     : {
                            'async'            : False,
                            'parseError'       : XMLDOMParseError.XMLDOMParseError(),
                          },
            'funcattrs' : {},
            'methods'   : {
                            'loadXML'          : MicrosoftXMLDOM.loadXML,
                            'createElement'    : MicrosoftXMLDOM.createElement,
                          }
        },


        # MicrosoftXMLHTTP
        {
            'id'        : (),
            'name'      : ( 'msxml2.xmlhttp', 'microsoft.xmlhttp', 'msxml2.xmlhttp.6.0'),
            'attrs'     : {
                            'bstrMethod'            : '',
                            'bstrUrl'               : '',
                            'varAsync'              : True,
                            'varUser'               : None,
                            'varPassword'           : None,
                            'status'                : 200,
                            'statusText'            : '200',
                            'requestHeaders'        : {},
                            'responseHeaders'       : {},
                            'responseBody'          : '',
                            'responseText'          : '',
                            'responseXML'           : '',
                            'readyState'            : 4,
                            'onerror'               : None,
                            'onload'                : None,
                            'onprogress'            : None,
                            'onreadystatechange'    : None,
                            'withCredentials'       : False,
                          },
            'funcattrs' : {},
            'methods'   : {
                            'abort'                 : MicrosoftXMLHTTP.abort,
                            'open'                  : MicrosoftXMLHTTP.open,
                            'send'                  : MicrosoftXMLHTTP.send,
                            'setRequestHeader'      : MicrosoftXMLHTTP.setRequestHeader,
                            'getResponseHeader'     : MicrosoftXMLHTTP.getResponseHeader,
                            'getAllResponseHeaders' : MicrosoftXMLHTTP.getAllResponseHeaders,
                            'overrideMimeType'      : MicrosoftXMLHTTP.overrideMimeType,
                          }
        },

        # Move
        {
            'id'        : ( '6054D082-355D-4B47-B77C-36A778899F48', ),
            'name'      : ( 'qmpupgrade.upgrade.1', ),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'Upgrade'       :   Move.Upgrade,
                          }
        },

        # MSRICHTXT
        {
            'id'        : ( '3B7C8860-D78F-101B-B9B5-04021C009402',
                            'B617B991-A767-4F05-99BA-AC6FCABB102E' ),
            'name'      : (),
            'attrs'     : { 'Text'          : '' },
            'funcattrs' : {},
            'methods'   : {
                            'SaveFile'      : MSRICHTXT.SaveFile,
                          }
        },

        # MSVFP
        {
            'id'        : ( 'A7CD2320-6117-11D7-8096-0050042A4CD2',
                            '008B6010-1F3D-11D1-B0C8-00A0C9055D74', ),
            'name'      : (),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'foxcommand'    : MSVFP.foxcommand,
                            'FoxCommand'    : MSVFP.foxcommand,
                            'DoCmd'         : MSVFP.foxcommand,
                            'docmd'         : MSVFP.foxcommand,
                          }
        },

        # MSXML2.DOMDocument
        {
            'id'        : ( 'F6D90F11-9C73-11D3-B32E-00C04F990BB4', ),
            'name'      : ( 'msxml2.domdocument', 'msxml2.domdocument.6.0'),
            'attrs'     : {
                            'object'        : MSXML2DOMDocument,
                          },
            'funcattrs' : {},
            'methods'   : {
                            'definition'    : MSXML2DOMDocument.definition,
                          },
        },

        # MyspaceUploader
        {
            'id'        : ( '48DD0448-9209-4F81-9F6D-D83562940134', ),
            'name'      : ( 'aurigma.imageuploader.4.1', ),
            'attrs'     : {
                            'Action'        : ''},
            'funcattrs' : {
                            'Action'        : MyspaceUploader.SetAction,
                          },
            'methods'   : {
                            'SetAction'     : MyspaceUploader.SetAction,
                          }
        },

        # NamoInstaller
        # FIXME DOWNLOAD
        {
            'id'        : ( 'AF465549-1D22-4140-A273-386FA8877E0A',  ),
            'name'      : (),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'Install'       : NamoInstaller.Install,
                          }
        },

        # NCTAudioFile2
        {
            'id'        : ( '77829F14-D911-40FF-A2F0-D11DB8D6D0BC', ),
            'name'      : ( 'nctaudiofile2.audiofile.1', ),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'SetFormatLikeSample'   :   NCTAudioFile2.SetFormatLikeSample,
                          }
        },

        # NeoTracePro
        {
            'id'        : ( '3E1DD897-F300-486C-BEAF-711183773554',  ),
            'name'      : (),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'TraceTarget'           : NeoTracePro.TraceTarget,
                          }
        },

        # NessusScanCtrl
        {
            'id'        : ( 'A47D5315-321D-4DEE-9DB3-18438023193B', ),
            'name'      : (),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'deleteReport'          : NessusScanCtrl.deleteReport,
                            'deleteNessusRC'        : NessusScanCtrl.deleteNessusRC,
                            'saveNessusRC'          : NessusScanCtrl.saveNessusRC,
                            'addsetConfig'          : NessusScanCtrl.addsetConfig,
                          }
        },

        # OfficeOCX
        {
            'id'        : ( '97AF4A45-49BE-4485-9F55-91AB40F288F2', # Office
                            '97AF4A45-49BE-4485-9F55-91AB40F22B92', # PowerPoint
                            '97AF4A45-49BE-4485-9F55-91AB40F22BF2', # Word
                            '18A295DA-088E-42D1-BE31-5028D7F9B965', # Excel
                          ),
            'name'      : (),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'OpenWebFile'           : OfficeOCX.OpenWebFile,
                          }
        },

        # OurgameGLWorld
        {
            'id'        : ( '61F5C358-60FB-4A23-A312-D2B556620F20', ),
            'name'      : ( 'hangameplugincn18.hangameplugincn18.1'),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'hgs_startGame'         : OurgameGLWorld.hgs_startGame,
                            'hgs_startNotify'       : OurgameGLWorld.hgs_startNotify,
                          }
        },

        # PPlayer
        {
            'id'        : ( 'F3E70CEA-956E-49CC-B444-73AFE593AD7F',
                            '5EC7C511-CD0F-42E6-830C-1BD9882F3458', ),
            'name'      : ( 'pplayer.xpplayer.1', ),
            'attrs'     : {
                            'FlvPlayerUrl'      : '',
                            'Logo'              : '',
                          },
            'funcattrs' : {
                            'FlvPlayerUrl'      : PPlayer.SetFlvPlayerUrl,
                            'Logo'              : PPlayer.SetLogo,
                          },
            'methods'   : {
                            'DownURL2'          : PPlayer.DownURL2,
                            'SetFlvPlayerUrl'   : PPlayer.SetFlvPlayerUrl,
                            'SetLogo'           : PPlayer.SetLogo,
                          }
        },

        # PTZCamPanel
        {
            'id'        : ( 'A86934DA-C3D6-4C1C-BD83-CA4F14B362DE', ),
            'name'      : (),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'ConnectServer'     : PTZCamPanel.ConnectServer,
                          }
        },

        # QuantumStreaming
        {
            'id'        : ( 'E473A65C-8087-49A3-AFFD-C5BC4A10669B', ),
            'name'      : ( 'qsp2ie.qsp2ie', ),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'UploadLogs'        : QuantumStreaming.UploadLogs,
                          }
        },

        # QvodCtrl
        {
            'id'        : ( 'F3D0D36F-23F8-4682-A195-74C92B03D4AF', ),
            'name'      : ( 'qvodinsert.qvodctrl.1', ),
            'attrs'     : {
                            'URL'               : '',
                            'url'               : '',
                          },
            'funcattrs' : {
                            'URL'               : QvodCtrl.SetURL,
                            'url'               : QvodCtrl.SetURL,
                          },
            'methods'   : {
                            'SetURL'            : QvodCtrl.SetURL,
                          }
        },

        # RDS.DataSpace
        {
            'id'        : ( 'BD96C556-65A3-11D0-983A-00C04FC29E36',
                            'BD96C556-65A3-11D0-983A-00C04FC29E30',
                            'AB9BCEDD-EC7E-47E1-9322-D4A210617116',
                            '0006F033-0000-0000-C000-000000000046',
                            '0006F03A-0000-0000-C000-000000000046',
                            '7F5B7F63-F06F-4331-8A26-339E03C0AE3D',
                            '06723E09-F4C2-43c8-8358-09FCD1DB0766',
                            '639F725F-1B2D-4831-A9FD-874847682010',
                            'BA018599-1DB3-44f9-83B4-461454C84BF8',
                            'D0C07D56-7C69-43F1-B4A0-25F5A11FAB19',
                            'E8CCCDDF-CA28-496b-B050-6C07C962476B',
                            '6E32070A-766D-4EE6-879C-DC1FA91D2FC3',
                            '6414512B-B978-451D-A0D8-FCFDF33E833C'),
            'name'      : ( 'rdsdataspace', 'rds.dataspace', ),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'CreateObject'  : RDSDataSpace.CreateObject,
                          }
        },

        # RealPlayer
        {
            'id'        : ( 'FDC7A535-4070-4B92-A0EA-D9994BCC0DC5',
                            '2F542A2E-EDC9-4BF7-8CB1-87C9919F7F93',
                            '0FDF6D6B-D672-463B-846E-C6FF49109662',
                            '224E833B-2CC6-42D9-AE39-90B6A38A4FA2',
                            '3B46067C-FD87-49B6-8DDD-12F0D687035F',
                            '3B5E0503-DE28-4BE8-919C-76E0E894A3C2',
                            '44CCBCEB-BA7E-4C99-A078-9F683832D493',
                            'A1A41E11-91DB-4461-95CD-0C02327FD934',
                            'CFCDAA03-8BE4-11CF-B84B-0020AFBBCCFA', ),
            'name'      : ( 'ierpctl.ierpctl', 'ierpctl.ierpctl.1', ),
            'attrs'     : {
                            'Console'               : '',
                          },
            'funcattrs' : {
                            'Console'               : RealPlayer.SetConsole,
                          },
            'methods'   : {
                            'DoAutoUpdateRequest'   : RealPlayer.DoAutoUpdateRequest,
                            'PlayerProperty'        : RealPlayer.PlayerProperty,
                            'Import'                : RealPlayer.Import,
                            'SetConsole'            : RealPlayer.SetConsole,
                          }
        },

        # RediffBolDownloaderAttack
        {
            'id'        : ( 'BADA82CB-BF48-4D76-9611-78E2C6F49F03', ),
            'name'      : (),
            'attrs'     : {
                            'url'       : '',
                            'start'     : '',
                            'fontsize'  : 14,
                            'barcolor'  : 'EE4E00',
                          },
            'funcattrs' : {
                            'url'       : RediffBolDownloaderAttack.Seturl,
                          },
            'methods'   : {
                            'Seturl'    : RediffBolDownloaderAttack.Seturl,
                          }
        },

        # RegistryPro
        {
            'id'        : ( 'D5C839EB-DA84-4F98-9D42-2074C2EE9EFC', ),
            'name'      : (),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'DeleteKey'     : RegistryPro.DeleteKey,
                            'About'         : RegistryPro.About,
                          }
        },

        # RisingScanner
        {
            'id'        : ( 'E4E2F180-CB8B-4DE9-ACBB-DA745D3BA153', ),
            'name'      : (),
            'attrs'     : {
                            'BaseURL'       : '',
                            'Encardid'      : '',
                          },
            'funcattrs' : {},
            'methods'   : {
                            'UpdateEngine'  : RisingScanner.UpdateEngine,
                          }
        },

        # RtspVaPgCtrl
        {
            'id'        : ( '361E6B79-4A69-4376-B0F2-3D1EBEE9D7E2', ),
            'name'      : ( 'rtspvapgdecoder.rtspvapgctrl.1', ),
            'attrs'     : {
                            'MP4Prefix'         : '',
                          },
            'funcattrs' : {
                            'MP4Prefix'         : RtspVaPgCtrl.SetMP4Prefix,
                          },
            'methods'   : {
                            'SetMP4Prefix'      : RtspVaPgCtrl.SetMP4Prefix,
                          }
        },

        # Scripting.FileSystemObject
        {
            'id'        : (),
            'name'      : ( 'scripting.filesystemobject', ),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'OpenTextFile'      : ScriptingFileSystemObject.OpenTextFile,
                            'Write'             : ScriptingFileSystemObject.Write,
                            'Close'             : ScriptingFileSystemObject.Close,
                            'BuildPath'         : ScriptingFileSystemObject.BuildPath,
                            'GetSpecialFolder'  : ScriptingFileSystemObject.GetSpecialFolder,
                            'GetTempName'       : ScriptingFileSystemObject.GetTempName,
                            'FileExists'        : ScriptingFileSystemObject.FileExists,
                            'CreateTextFile'    : ScriptingFileSystemObject.CreateTextFile,
                            'GetFile'           : ScriptingFileSystemObject.GetFile,
                            'GetExtensionName'  : ScriptingFileSystemObject.GetExtensionName,
                          },
        },

        # Shell.Application
        {
            'id'        : (),
            'name'      : ( 'shell.application', ),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'ShellExecute'  : ShellApplication.ShellExecute,
                            'shellexecute'  : ShellApplication.ShellExecute,
                          }
        },

        # Shockwave
        {
            'id'        : ( '233C1507-6A77-46A4-9443-F871F945D258', ),
            'name'      : ( 'swctl.swctl', 'swctl.swctl.8',  ),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'ShockwaveVersion'   : Shockwave.ShockwaveVersion,
                          }
        },

        # ShockwaveFlash.ShockwaveFlash.9
        {
            'id'        : (),
            'name'      : ( 'shockwaveflash.shockwaveflash.9', ),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'GetVariable'   : ShockwaveFlash9.GetVariable,
                          }
        },

        # ShockwaveFlash.ShockwaveFlash.10
        {
            'id'        : (),
            'name'      : ( 'shockwaveflash.shockwaveflash', 'shockwaveflash.shockwaveflash.10', ),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'GetVariable'   : ShockwaveFlash10.GetVariable,
                          }
        },

        # ShockwaveFlash.ShockwaveFlash.11
        {
            'id'        : (),
            'name'      : ( 'shockwaveflash.shockwaveflash.11', ),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'GetVariable'   : ShockwaveFlash11.GetVariable,
                          }
        },

        # ShockwaveFlash.ShockwaveFlash.12
        {
            'id'        : (),
            'name'      : ( 'shockwaveflash.shockwaveflash.12', ),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'GetVariable'   : ShockwaveFlash12.GetVariable,
                          }
        },

        # SilverLight
        {
            'id'        : (),
            'name'      : ( 'agcontrol.agcontrol', ),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'isVersionSupported' : SilverLight.isVersionSupported,
                          }
        },

        # SinaDLoader
        # FIXME Download
        {
            'id'        : ( '78ABDC59-D8E7-44D3-9A76-9A0918C52B4A', ),
            'name'      : ( 'downloader.dloader.1', ),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'DownloadAndInstall'   : SinaDLoader.DownloadAndInstall,
                          }
        },


        # SnapshotViewer
        {
            'id'        : ( 'F0E42D60-368C-11D0-AD81-00A0C90DC8D9',
                            'F2175210-368C-11D0-AD81-00A0C90DC8D9' ),
            'name'      : ( 'snpvw.snapshot viewer control.1'),
            'attrs'     : {
                            'SnapshotPath'          : '',
                            'CompressedPath'        : '',
                          },
            'funcattrs' : {},
            'methods'   : {
                            'PrintSnapshot'         : SnapshotViewer.PrintSnapshot,
                          }
        },

        # SonicWallNetExtenderAddRouteEntry
        {
            'id'        : ( '6EEFD7B1-B26C-440D-B55A-1EC677189F30', ),
            'name'      : (),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'AddRouteEntry'   : SonicWallNetExtenderAddRouteEntry.AddRouteEntry,
                          }
        },

        # Spreadsheet
        {
            'id'        : ( '0002E543-0000-0000-C000-000000000046',
                            '0002E55B-0000-0000-C000-000000000046' ),
            'name'      : ( 'owc10.spreadsheet',
                            'owc11.spreadsheet'),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'Evaluate'              : Spreadsheet.Evaluate,
                            '_Evaluate'             : Spreadsheet._Evaluate,
                          }
        },

        # SSReaderPdg2
        {
            'id'        : ( '7F5E27CE-4A5C-11D3-9232-0000B48A05B2', ),
            'name'      : ( 'pdg2', ),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'Register'              : SSReaderPdg2.Register,
                            'LoadPage'              : SSReaderPdg2.LoadPage,
                          }
        },

        # StormConfig
        {
            'id'        : ( 'BD103B2B-30FB-4F1E-8C17-D8F6AADBCC05', ),
            'name'      : ( 'config', ),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'SetAttributeValue'     : StormConfig.SetAttributeValue,
                          }
        },

        # StormMps
        {
            'id'        : ( '6BE52E1D-E586-474F-A6E2-1A85A9B4D9FB', ),
            'name'      : ( 'mps.stormplayer.1', ),
            'attrs'     : {
                            'URL'                   : '',
                            'backImage'             : '',
                            'titleImage'            : '',

                          },
            'funcattrs' : {
                            'URL'                   : StormMps.SetURL,
                            'backImage'             : StormMps.SetbackImage,
                            'titleImage'            : StormMps.SettitleImage,
                          },
            'methods'   : {
                            'advancedOpen'          : StormMps.advancedOpen,
                            'isDVDPath'             : StormMps.isDVDPath,
                            'rawParse'              : StormMps.rawParse,
                            'OnBeforeVideoDownload' : StormMps.OnBeforeVideoDownload,
                            'SetURL'                : StormMps.SetURL,
                            'SetbackImage'          : StormMps.SetbackImage,
                            'SettitleImage'         : StormMps.SettitleImage,
                          }
        },

        # SymantecAppStream
        {
            'id'        : ( '3356DB7C-58A7-11D4-AA5C-006097314BF8', ),
            'name'      : (),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'installAppMgr'     : SymantecAppStream.installAppMgr,
                          }
        },

        # SymantecBackupExec
        {
            'id'        : ( '22ACD16F-99EB-11D2-9BB3-00400561D975', ),
            'name'      : ( 'pvatlcalendar.pvcalendar.1', ),
            'attrs'     : {
                            '_DOWText0'         : '',
                            '_DOWText6'         : '',
                            '_MonthText0'       : '',
                            '_MonthText11'      : '',
                          },
            'funcattrs' : {
                            '_DOWText0'         : SymantecBackupExec.Set_DOWText0,
                            '_DOWText6'         : SymantecBackupExec.Set_DOWText6,
                            '_MonthText0'       : SymantecBackupExec.Set_MonthText0,
                            '_MonthText11'      : SymantecBackupExec.Set_MonthText11,
                          },
            'methods'   : {
                            'Save'              : SymantecBackupExec.Save,
                            'Set_DOWText0'      : SymantecBackupExec.Set_DOWText0,
                            'Set_DOWText6'      : SymantecBackupExec.Set_DOWText6,
                            'Set_MonthText0'    : SymantecBackupExec.Set_MonthText0,
                            'Set_MonthText11'   : SymantecBackupExec.Set_MonthText11,
                          }
        },

        # StreamAudioChainCast
        {
            'id'        : ( '2253F320-AB68-4A07-917D-4F12D8884A06', ),
            'name'      : ( 'ccpm.proxymanager.1'),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'InternalTuneIn'    : StreamAudioChainCast.InternalTuneIn,
                          }
        },

        # Toshiba
        {
            'id'        : ( 'AD315309-EA00-45AE-9E8E-B6A61CE6B974', ),
            'name'      : ( 'meipcamx.recordsend.1', ),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'SetPort'               : Toshiba.SetPort,
                            'SetIpAddress'          : Toshiba.SetIpAddress,
                          }
        },

        # UniversalUpload
        {
            'id'        : ( '04FD48E6-0712-4937-B09E-F3D285B11D82', ),
            'name'      : (),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'RemoveFileOrDir'       : UniversalUpload.RemoveFileOrDir,
                          }
        },

        # UUSeeUpdate
        {
            'id'        : ( '2CACD7BB-1C59-4BBB-8E81-6E83F82C813B', ),
            'name'      : ( 'uuupgrade.uuupgradectrl.1', ),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'Update'                : UUSeeUpdate.Update,
                          }
        },

        # VisualStudio.DTE.8.0
        {
            'id'        : ( 'BA018599-1DB3-44F9-83B4-461454C84BF8', ),
            'name'      : ( ),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'CreateObject'  : VisualStudioDTE80.CreateObject,
                          }
        },

        # VLC
        {
            'id'        : ( 'E23FE9C6-778E-49D4-B537-38FCDE4887D8', ),
            'name'      : (),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'getVariable'           : VLC.getVariable,
                            'setVariable'           : VLC.setVariable,
                            'addTarget'             : VLC.addTarget,
                          }
        },

        # VsaIDE.DTE
        {
            'id'        : ( 'E8CCCDDF-CA28-496B-B050-6C07C962476B', ),
            'name'      : ( ),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'CreateObject'  : VsaIDEDTE.CreateObject,
                          }
        },

        # VsmIDE.DTE
        {
            'id'        : ( '06723E09-F4C2-43C8-8358-09FCD1DB0766', ),
            'name'      : ( ),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'CreateObject'  : VsmIDEDTE.CreateObject,
                          }
        },

        # WebViewFolderIcon
        {
            'id'        : (),
            'name'      : ( 'webviewfoldericon.webviewfoldericon.1', ),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'setSlice'              : WebViewFolderIcon.setSlice,
                          }
        },

        # WindowsMediaPlayer
        {
            'id'        : ( '22D6F312-B0F6-11D0-94AB-0080C74C7E95', ),
            'name'      : (),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'Play'  : WindowsMediaPlayer.Play,
                          },
        },

        # WinZip
        {
            'id'        : ( 'A09AE68F-B14D-43ED-B713-BA413F034904', ),
            'name'      : ( 'wzfileview.fileviewctrl.61'),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'CreateNewFolderFromName'   : WinZip.CreateNewFolderFromName,
                          }
        },

        # WMEncProfileManager
        {
            'id'        : ( 'A8D3AD02-7508-4004-B2E9-AD33F087F43C', ),
            'name'      : ( 'wmencprofilemanager', ),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'GetDetailsString'          : WMEncProfileManager.GetDetailsString,
                          }
        },

        # WMP
        {
            'id'        : ( '6BF52A52-394A-11D3-B153-00C04F79FAA6', ),
            'name'      : (),
            'attrs'     : { 'versionInfo'           : 10},
            'funcattrs' : {},
            'methods'   : {
                            'openPlayer'            : WMP.openPlayer,
                          }
        },

        # WScriptShell
        {
            'id'        : (),
            'name'      : 'wscript.shell',
            'attrs'     : {
                            'scriptFullName'            : '',
                          },
            'funcattrs' : {},
            'methods'   :
                          {
                            'Run'                       : WScriptShell.Run,
                            '_doRun'                    : WScriptShell._doRun,
                            'Environment'               : WScriptShell.Environment,
                            'ExpandEnvironmentStrings'  : WScriptShell.ExpandEnvironmentStrings,
                            'CreateObject'              : WScriptShell.CreateObject,
                            'Sleep'                     : WScriptShell.Sleep,
                            'Quit'                      : WScriptShell.Quit,
                            'Echo'                      : WScriptShell.Echo,
                            'valueOf'                   : WScriptShell.valueOf,
                            'toString'                  : WScriptShell.toString,
                            'SpecialFolders'            : WScriptShell.SpecialFolders,
                            'CreateShortcut'            : WScriptShell.CreateShortcut,
                            'RegRead'                   : WScriptShell.RegRead,
                            'RegWrite'                  : WScriptShell.RegWrite,
                          }
        },

        # WScriptShortcut
        {
        'id'        : (),
        'name'      : ( 'wscript.shortcut'),
        'attrs'     : {
                        'FullName'              : '',
                        'TargetPath'            : '',
                        'Description'           : '',
                        'Hotkey'                : '',
                        'IconLocation'          : '',
                        'RelativePath'          : '',
                        'WindowStyle'           : 1,
                        'WorkingDirectory'      : '',
                      },
        'funcattrs' : {},
        'methods'   : {
                        'save'                  : WScriptShortcut.save,
                      }
        },

        # WScriptNetwork
        {
        'id'        : (),
        'name'      : ('wscript.network'),
        'attrs'     : {},
        'funcattrs' : {
                        'ComputerName'          : WScriptNetwork.GetComputerName,
                        'UserDomain'            : WScriptNetwork.GetUserDomain,
                        'UserName'              : WScriptNetwork.GetUserName,
        },
        'methods'   : {
                        'EnumPrinterConnections': WScriptNetwork.EnumPrinterConnections,
                        'EnumNetworkDrives'     : WScriptNetwork.EnumNetworkDrives,
                        'GetComputerName'       : WScriptNetwork.GetComputerName,
                        'GetUserDomain'         : WScriptNetwork.GetUserDomain,
                        'GetUserName'           : WScriptNetwork.GetUserName,
                      }
        },

        # XUpload
        {
            'id'        : ( 'E87F6C8E-16C0-11D3-BEF7-009027438003', ),
            'name'      : (),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'AddFolder'             : XUpload.AddFolder,
                            'AddFile'               : XUpload.AddFile,
                          }
        },

        # YahooJukebox
        {
            'id'        : ( '22FD7C0A-850C-4A53-9821-0B0915C96139',
                            '5F810AFC-BB5F-4416-BE63-E01DD117BD6C'),
            'name'      : (),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'AddBitmap'             : YahooJukebox.AddBitmap,
                            'AddButton'             : YahooJukebox.AddButton,
                            'AddImage'              : YahooJukebox.AddImage,
                          }
        },

        # YahooMessengerCyft
        {
            'id'        : ( '24F3EAD6-8B87-4C1A-97DA-71C126BDA08F', ),
            'name'      : (),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'GetFile'               : YahooMessengerCyft.GetFile,
                          }
        },

        # YahooMessengerYVerInfo
        {
            'id'        : ( 'D5184A39-CBDF-4A4F-AC1A-7A45A852C883', ),
            'name'      : (),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'fvcom'                 : YahooMessengerYVerInfo.fvcom,
                            'info'                  : YahooMessengerYVerInfo.info,
                          }
        },

        # YahooMessengerYwcvwr
        {
            'id'        : ( '9D39223E-AE8E-11D4-8FD3-00D0B7730277',
                            'DCE2F8B1-A520-11D4-8FD0-00D0B7730277',
                            '7EC7B6C5-25BD-4586-A641-D2ACBB6629DD'),
            'name'      : (),
            'attrs'     : {
                            'server'                : ''
                          },
            'funcattrs' : {
                            'server'                : YahooMessengerYwcvwr.Setserver,
                          },
            'methods'   : {
                            'Setserver'             : YahooMessengerYwcvwr.Setserver,
                            'GetComponentVersion'   : YahooMessengerYwcvwr.GetComponentVersion,
                            'initialize'            : YahooMessengerYwcvwr.initialize,
                            'send'                  : YahooMessengerYwcvwr.send,
                            'receive'               : YahooMessengerYwcvwr.receive,
                          }
        },

        # ZenturiProgramCheckerAttack
        {
            'id'        : ( '59DBDDA6-9A80-42A4-B824-9BC50CC172F5', ),
            'name'      : (),
            'attrs'     : {},
            'funcattrs' : {},
            'methods'   : {
                            'DownloadFile'          : ZenturiProgramCheckerAttack.DownloadFile,
                            'DebugMsgLog'           : ZenturiProgramCheckerAttack.DebugMsgLog,
                            'NavigateUrl'           : ZenturiProgramCheckerAttack.NavigateUrl,
                          }
        },
]



# NCTsoft Products NCTAudioFile2 ActiveX Control
# CVE-2007-0018

import logging

log = logging.getLogger("Thug")


def SetFormatLikeSample(self, arg):
    if len(arg) > 4000:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "NCTAudioFile2 ActiveX",
                                          "Overflow in SetFormatLikeSample",
                                          cve = "CVE-2007-0018")

        log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2007-0018", None)



# Move Networks Upgrade Manager 1.x
# CVE-NOMATCH

import logging

log = logging.getLogger("Thug")


def Upgrade(self, arg0, arg1, arg2, arg3):
    if len(arg0) > 6000:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "Move Networks Upgrade Manager ActiveX",
                                          "Overflow in Upgrade")



#!/usr/bin/env python

import logging

log = logging.getLogger("Thug")


# Introduced in DOM Level 2
class EventListener(object):
    def __init__(self):
        pass

    def handleEvent(self, evt):
        log.debug('handleEvent(%s)', evt)



#!/usr/bin/env python
#
# Mapper.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA
#
# Author:   Thorsten Sick <thorsten.sick@avira.com> from Avira
#           (developed for the iTES Project http://ites-project.org)
#
# Changes:
#           - pydot support                 (Angelo Dell'Aera <angelo.dellaera@honeynet.org>)
#           - Replace pydot with pygraphviz (Angelo Dell'Aera <angelo.dellaera@honeynet.org>)

import os
import json
import fnmatch
import pygraphviz

try:
    import urllib.parse as urlparse
except ImportError:
    import urlparse


class DictDiffer(object):
    """
    Calculate the difference between two dictionaries as:
    (1) items added
    (2) items removed
    (3) keys same in both but changed values
    (4) keys same in both and unchanged values
    """
    def __init__(self, current_dict, past_dict):
        self.current_dict = current_dict
        self.past_dict    = past_dict
        self.set_current  = set(current_dict.keys())
        self.set_past     = set(past_dict.keys())
        self.intersect    = self.set_current.intersection(self.set_past)

    def added(self):
        return self.set_current - self.intersect

    def removed(self):
        return self.set_past - self.intersect

    def changed(self):
        return set(o for o in self.intersect if self.past_dict[o] != self.current_dict[o])

    def unchanged(self):
        return set(o for o in self.intersect if self.past_dict[o] == self.current_dict[o])

    def anychange(self):
        if len(self.added()):
            return True

        if len(self.removed()):
            return True

        if len(self.changed()):
            return True

        return False


class Mapper(object):
    """
        Map URL relationships
    """

    markup_types = ("text/html",
                    "text/xml",
                    "text/css", )

    image_types  = ("image/", )

    exec_types   = ("application/javascript",
                    "text/javascript",
                    "application/x-javascript")

    def __init__(self, resdir, simplify = False):
        """
        @resdir     : Directory to store the result svg in
        @simplify   : Reduce the urls to server names
        """

        self.simplify = simplify

        self.data = {
                        "locations"   : [],
                        "connections" : []
                    }

        self.first_track = True   # flag indicating that we did not follow a track yet
        self.__init_graph(resdir)

    def __init_graph(self, resdir):
        graphdir     = os.path.abspath(os.path.join(resdir, os.pardir))
        self.svgfile = os.path.join(graphdir, "graph.svg")
        self.graph   = pygraphviz.AGraph(strict   = False,
                                         directed = True,
                                         rankdir  = 'LR')

    def check_content_type(self, loc, t):
        return loc["content-type"] and loc["content-type"].lower().startswith(t)

    def check_types(self, loc, _types):
        for t in _types:
            if self.check_content_type(loc, t):
                return True

        return False

    def check_markup(self, loc):
        return self.check_types(loc, self.markup_types)

    def check_image(self, loc):
        return self.check_types(loc, self.image_types)

    def check_exec(self, loc):
        return self.check_types(loc, self.exec_types)

    def get_shape(self, loc):
        # Markup
        if self.check_markup(loc):
            return "box"

        # Images
        if self.check_image(loc):
            return "oval"

        # Executable stuff
        if self.check_exec(loc):
            return "hexagon"

        return None

    def get_fillcolor(self, loc):
        if "error" in loc["flags"]:
            return "orange"

        return None

    def get_color(self, con):
        if "method" in ["iframe"]:
            return "orange"

        return None

    def normalize_url(self, url):
        if url.endswith("/"):
            return url[:-1]

        return url

    def dot_from_data(self):
        # Create dot from data
        if "locations" in self.data:
            for loc in self.data["locations"]:
                if loc["display"] is False:
                    continue

                url = self.normalize_url(loc["url"])

                self.graph.add_node(url)

                node = self.graph.get_node(url)

                shape = self.get_shape(loc)
                if shape:
                    node.attr['shape'] = shape

                fillcolor = self.get_fillcolor(loc)
                if fillcolor:
                    node.attr['style']     = 'filled'
                    node.attr['fillcolor'] = fillcolor

        if "connections" in self.data:
            # Add edges
            for con in self.data["connections"]:
                if con["display"] is False:
                    continue

                _s = self.normalize_url(con["source"])
                source = self.graph.get_node(_s)
                if not source:
                    source = _s

                _d = self.normalize_url(con["destination"])
                destination = self.graph.get_node(_d)
                if not destination:
                    destination = _d

                self.graph.add_edge(source, destination)

                edge = self.graph.get_edge(source, destination)
                edge.attr['label'] = con['method']

                color = self.get_color(con)
                if color:
                    edge.attr['color'] = color

    def add_location(self, loc):
        """
            Add location information to location data
        """
        loc["display"] = True

        if self.simplify:
            url = urlparse.urlparse(loc["url"]).netloc
            if url:
                loc["url"] = url

        for a in self.data["locations"]:
            d = DictDiffer(a, loc)
            if not d.anychange():
                return

        self.data["locations"].append(loc)

    def add_weak_location(self, url):
        """
        Generate a weak location for the given url.
        """

        for l in self.data["locations"]:
            if l["url"] == url:
                return

        loc = {'mimetype'       : '',
               'url'            : url,
               'size'           : 0,
               'flags'          : {},
               'sha256'         : None,
               'content-type'   : None,
               'display'        : True,
               'md5'            : None}

        self.add_location(loc)

    def add_connection(self, con):
        """
            Add connection information to connection data
        """
        con["display"] = True

        if self.simplify:
            url = urlparse.urlparse(con["source"]).netloc
            if url:
                con["source"] = url

            url = urlparse.urlparse(con["destination"]).netloc
            if url:
                con["destination"] = url

        self.add_weak_location(con["source"])
        self.add_weak_location(con["destination"])

        for a in self.data["connections"]:
            d = DictDiffer(a, con)
            if not d.anychange():
                return

        self.data["connections"].append(con)

    def add_data(self, data):
        # Add nodes
        if "locations" in data:
            for loc in data["locations"]:
                self.add_location(loc)

        if "connections" in data:
            for con in data["connections"]:
                self.add_connection(con)

    def add_file(self, filename):
        """
            Add data file
        """
        try:
            self.add_data(json.load(open(filename, "r")))
        except ValueError:
            pass

    def write_svg(self):
        """
            Create SVG file
        """
        self.dot_from_data()

        try:
            self.graph.layout(prog = 'dot')
            self.graph.draw(self.svgfile, format = 'svg')
        except:  # pylint:disable=bare-except
            pass

    def activate(self, conto):
        """
            Iterate through data and set display for hot connections
        """

        tofix = []

        for c in self.data["connections"]:
            if c["display"] is False and c["destination"] == conto:
                c["display"] = True
                tofix.append(c["source"])

        for l in self.data["locations"]:
            if l["url"] == conto or l["url"] in tofix:
                l["display"] = True

        for t in tofix:
            self.activate(t)

    def follow_track(self, end):
        """
            Follow the track between entry point of the analysis and the exploit URL.
            Remove all non-relevant stuff

            @end: end url to track the connections to
        """

        if self.first_track:
            for con in self.data["connections"]:
                con["display"] = False

            for loc in self.data["locations"]:
                loc["display"] = False

        self.first_track = False
        self.activate(end)

    def write_text(self):
        """
            Return text representation
        """

        res = ""
        for con in self.data["connections"]:
            if con["display"]:
                res += "%s -- %s --> %s \n" % (str(con["source"]), str(con["method"]), str(con["destination"]))
        return res


def allFiles(root, patterns = "*", single_level = False, yield_folders = False):
    """
        Walk files
    """

    patterns = patterns.split(";")
    for path, subdirs, files in os.walk(root):
        if yield_folders:
            files.extend(subdirs)

        files.sort()

        for name in files:
            for pattern in patterns:
                if fnmatch.fnmatch(name, pattern):
                    yield os.path.join(path, name)
                    break

        if single_level:
            break

if __name__ == "__main__":
    import argparse

    parser = argparse.ArgumentParser(description = 'Receives jobs and starts Thug to process them')
    parser.add_argument('--resdir', help = 'Result dir', default = ".")
    parser.add_argument('--source', help = 'Source file or dir', default = "avlog.json")
    parser.add_argument('--simplify', help = 'Reduce the URLs to servernames', default = False, action = "store_true")
    parser.add_argument('--tracks', type=str, nargs='+', help='URLs to track to', default = None)

    args = parser.parse_args()

    m = Mapper(args.resdir, simplify = args.simplify)
    if os.path.isdir(args.source):
        for afile in allFiles(args.source, "analysis.json"):
            m.add_file(afile)
    else:
        m.add_file(args.source)

    if args.tracks:
        for atrack in args.tracks:
            m.follow_track(atrack)

    m.write_svg()



#!/usr/bin/env python

import logging
log = logging.getLogger("Thug")


class CSSStyleDeclaration(object):
    def __init__(self, style):
        # self.props = dict([prop.strip().split(': ') for prop in style.split(';') if prop])
        self.props = dict()

        for prop in [p for p in style.split(';') if p]:
            k, v = prop.strip().split(':')
            self.props[k.strip()] = v.strip()

        for k, v in self.props.items():
            if v and v[0] == v[-1] and v[0] in ['"', "'"]:
                self.props[k] = v[1:-1]

    @property
    def cssText(self):
        return '; '.join(["%s: %s" % (k, v) for k, v in self.props.items()])

    def getPropertyValue(self, name):
        return self.props.get(name, '')

    def removeProperty(self, name):
        v = self.props.get(name, '')

        if v:
            del self.props[name]

        return v

    @property
    def length(self):
        return len(self.props)

    def item(self, index):
        if type(index) == str:
            if log.ThugOpts.Personality.isIE() and log.ThugOpts.Personality.browserMajorVersion < 7 and index in ('maxHeight', ):
                raise AttributeError(index)

            return self.props.get(index, '')

        if index < 0 or index >= len(self.props):
            return ''

        return self.props[self.props.keys()[index]]

    def __getattr__(self, name):
        if log.ThugOpts.Personality.isIE() and log.ThugOpts.Personality.browserMajorVersion < 7 and name in ('maxHeight', ):
            raise AttributeError(name)

        if hasattr(object, name):
            return object.__getattribute__(self, name)
        else:
            return object.__getattribute__(self, 'props').get(name, '')

    def __setattr__(self, name, value):
        if name == 'props':
            object.__setattr__(self, name, value)
        else:
            object.__getattribute__(self, 'props')[name] = value



#!/usr/bin/env python

from .HTMLElement import HTMLElement
from .attr_property import attr_property
from .text_property import text_property
from .compatibility import thug_long


class HTMLTextAreaElement(HTMLElement):
    def __init__(self, doc, tag):
        HTMLElement.__init__(self, doc, tag)

    defaultValue    = None

    @property
    def form(self):
        pass

    accessKey       = attr_property("accesskey")
    cols            = attr_property("cols", thug_long)
    disabled        = attr_property("disabled", bool)
    name            = attr_property("name")
    readOnly        = attr_property("readonly", bool)
    rows            = attr_property("rows", thug_long)
    tabIndex        = attr_property("tabindex", thug_long)
    value           = text_property()

    @property
    def type(self):
        return "textarea"

    def focus(self):
        pass

    def blur(self):
        pass

    def select(self):
        pass



#!/usr/bin/env python

from .HTMLElement import HTMLElement


class HTMLFieldSetElement(HTMLElement):
    def __init__(self, doc, tag):
        HTMLElement.__init__(self, doc, tag)

    @property
    def form(self):
        pass



#!/usr/bin/env python

from .Node import Node


class Entity(Node):
    @property
    def publicId(self):
        pass

    @property
    def systemId(self):
        pass

    @property
    def notationName(self):
        pass

    @property
    def nodeName(self):
        pass

    @property
    def nodeType(self):
        return Node.ENTITY_NODE

    @property
    def nodeValue(self):
        return None



#!/usr/bin/env python

from thug.DOM.JSClass import JSClass


class HTMLCollection(JSClass):
    def __init__(self, doc, nodes):
        self.doc   = doc
        self.nodes = nodes

    def __len__(self):
        return self.length

    def __getitem__(self, key):
        try:
            return self.item(int(key))
        except TypeError:
            return self.namedItem(str(key))

    @property
    def length(self):
        return len(self.nodes)

    def item(self, index):
        # from thug.DOM.W3C.DOMImplementation import DOMImplementation

        node = self.nodes[index]

        return node
        # return DOMImplementation.createHTMLElement(self.doc, node) if node else None

    def namedItem(self, name):
        from thug.DOM.W3C.DOMImplementation import DOMImplementation

        for node in self.nodes:
            if node.nodeName == name:
                return DOMImplementation.createHTMLElement(self.doc, node) if node else None

        return None



__all__ = [
            'AcroPDF',
            'AdodbStream',
            'AnswerWorks',
            'AolAmpX',
            'AolICQ',
            'AOLAttack',
            'BaiduBar',
            'BitDefender',
            'CABrightStor',
            'CGAgent',
            'Comodo',
            'ConnectAndEnterRoom',
            'CreativeSoftAttack',
            'DirectShow',
            'DivX',
            'DLinkMPEG',
            'Domino',
            'DPClient',
            'DVRHOSTWeb',
            'EnjoySAP',
            'FacebookPhotoUploader',
            'FileUploader',
            'GatewayWeblaunch',
            'GLIEDown2',
            'Gogago',
            'GomWeb',
            'HPInfo',
            'ICQToolbar',
            'IMWebControl',
            'InternetCleverSuite',
            'JavaDeploymentToolkit',
            'JetAudioDownloadFromMusicStore',
            'Kingsoft',
            'MacrovisionFlexNet',
            'MicrosoftWorks7Attack',
            'MicrosoftXMLDOM',
            'MicrosoftXMLHTTP',
            'Move',
            'MSRICHTXT',
            'MSVFP',
            'MSXML2DOMDocument',
            'MyspaceUploader',
            'NamoInstaller',
            'NCTAudioFile2',
            'NeoTracePro',
            'NessusScanCtrl',
            'OfficeOCX',
            'OurgameGLWorld',
            'PPlayer',
            'PTZCamPanel',
            'QuantumStreaming',
            'QvodCtrl',
            'RDSDataSpace',
            'RealPlayer',
            'RediffBolDownloaderAttack',
            'RegistryPro',
            'RisingScanner',
            'RtspVaPgCtrl',
            'ScriptingFileSystemObject',
            'ShellApplication',
            'Shockwave',
            'ShockwaveFlash9',
            'ShockwaveFlash10',
            'ShockwaveFlash11',
            'ShockwaveFlash12',
            'SilverLight',
            'SinaDLoader',
            'SnapshotViewer',
            'SonicWallNetExtenderAddRouteEntry',
            'Spreadsheet',
            'SSReaderPdg2',
            'StormConfig',
            'StormMps',
            'StreamAudioChainCast',
            'SymantecAppStream',
            'SymantecBackupExec',
            'Toshiba',
            'UniversalUpload',
            'UUSeeUpdate',
            'VisualStudioDTE80',
            'VLC',
            'VsaIDEDTE',
            'VsmIDEDTE',
            'WebViewFolderIcon',
            'WindowsMediaPlayer',
            'WinZip',
            'WMEncProfileManager',
            'WScriptShell',
            'WScriptShortcut',
            'WScriptNetwork',
            'WMP',
            'XMLDOMParseError',
            'XUpload',
            'YahooJukebox',
            'YahooMessengerCyft',
            'YahooMessengerYVerInfo',
            'YahooMessengerYwcvwr',
            'ZenturiProgramCheckerAttack'
]



#!/usr/bin/env python

from .HTMLCollection import HTMLCollection
from thug.DOM.W3C.NodeList import NodeList


class HTMLAllCollection(HTMLCollection):
    def __init__(self, doc, nodes):
        HTMLCollection.__init__(self, doc, nodes)

    def tags(self, name):
        s = [p for p in self.doc.find_all(name.lower())]
        return NodeList(self.doc, s)



#!/usr/bin/env python
#
# JSON.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA
#
# Author:   Thorsten Sick <thorsten.sick@avira.com> from Avira
#           (developed for the iTES Project http://ites-project.org)

import logging
import datetime
import os
import json

try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO

from .Mapper import Mapper
from .compatibility import thug_unicode

log = logging.getLogger("Thug")


class JSON(object):
    def __init__(self, thug_version, provider = False):
        self._tools = ({
                        'id'          : 'json-log',
                        'Name'        : 'Thug',
                        'Version'     : thug_version,
                        'Vendor'      : None,
                        'Organization': 'The Honeynet Project',
                       }, )

        self.associated_code = None
        self.object_pool     = None
        self.signatures      = list()
        self.cached_data     = None
        self.provider        = provider

        self.data = {
                        "url"         : None,
                        "timestamp"   : str(datetime.datetime.now()),
                        "logtype"     : "json-log",
                        "thug"        : {
                                        "version"            : thug_version,
                                        "personality" : {
                                            "useragent"      : log.ThugOpts.useragent
                                            },
                                        "plugins" : {
                                            "acropdf"        : self.get_vuln_module("acropdf"),
                                            "javaplugin"     : self.get_vuln_module("_javaplugin"),
                                            "shockwaveflash" : self.get_vuln_module("shockwave_flash")
                                            },
                                        "options" : {
                                            "local"          : log.ThugOpts.local,
                                            "nofetch"        : log.ThugOpts.no_fetch,
                                            "proxy"          : log.ThugOpts._proxy,
                                            "events"         : log.ThugOpts.events,
                                            "delay"          : log.ThugOpts.delay,
                                            "referer"        : log.ThugOpts.referer,
                                            "timeout"        : log.ThugOpts.timeout,
                                            "threshold"      : log.ThugOpts.threshold,
                                            "extensive"      : log.ThugOpts.extensive,
                                            },
                                        },
                        "behavior"    : [],
                        "code"        : [],
                        "files"       : [],
                        "connections" : [],
                        "locations"   : [],
                        "exploits"    : [],
                        "classifiers" : [],
                    }

    @property
    def json_enabled(self):
        return log.ThugOpts.json_logging or 'json' in log.ThugLogging.formats or self.provider

    def get_vuln_module(self, module):
        disabled = getattr(log.ThugVulnModules, "%s_disabled" % (module, ), True)
        if disabled:
            return "disabled"

        return getattr(log.ThugVulnModules, module)

    def fix(self, data):
        """
        Fix encoding of data

        @data  data to encode properly
        """
        try:
            enc = log.Encoding.detect(data)
            return data.decode(enc['encoding']).replace("\n", "").strip()
        except:  # pylint:disable=bare-except
            return thug_unicode(data).replace("\n", "").strip()

    def make_counter(self, p):
        _id = p
        while True:
            yield _id
            _id += 1

    def set_url(self, url):
        if not self.json_enabled:
            return

        self.data["url"] = self.fix(url)

    def add_code_snippet(self, snippet, language, relationship, method = "Dynamic Analysis"):
        if not self.json_enabled:
            return

        self.data["code"].append({"snippet"      : self.fix(snippet),
                                  "language"     : self.fix(language),
                                  "relationship" : self.fix(relationship),
                                  "method"       : self.fix(method)})

    def log_connection(self, source, destination, method, flags = None):
        """
        Log the connection (redirection, link) between two pages

        @source        The origin page
        @destination   The page the user is made to load next
        @method        Link, iframe, .... that moves the user from source to destination
        @flags         Additional information flags. Existing are: "exploit"
        """
        if not self.json_enabled:
            return

        if flags is None:
            flags = dict()

        if "exploit" in flags and flags["exploit"]:
            self.add_behavior_warn("[Exploit]  %s -- %s --> %s" % (source,
                                                                   method,
                                                                   destination, ))
        else:
            self.add_behavior_warn("%s -- %s --> %s" % (source,
                                                        method,
                                                        destination,))

        self.data["connections"].append({"source"       : self.fix(source),
                                         "destination"  : self.fix(destination),
                                         "method"       : method,
                                         "flags"        : flags})

    def log_location(self, url, data, flags = None):
        """
        Log file information for a given url

        @url    URL we fetched data from
        @data   File dictionary data
                    Keys:
                        - content     Content
                        - md5         MD5 checksum
                        - sha256      SHA-256 checksum
                        - fsize       Content size
                        - ctype       Content type (whatever the server says it is)
                        - mtype       Calculated MIME type

        @flags  Additional information flags (known flags: "error")
        """
        if not self.json_enabled:
            return

        if flags is None:
            flags = dict()

        self.data["locations"].append({"url"          : self.fix(url),
                                       "status"       : data.get("status", None),
                                       "content-type" : data.get("ctype", None),
                                       "md5"          : data.get("md5", None),
                                       "sha256"       : data.get("sha256", None),
                                       "flags"        : flags,
                                       "size"         : data.get("fsize", None),
                                       "mimetype"     : data.get("mtype", None)})

    def log_exploit_event(self, url, module, description, cve = None, data = None):
        """
        Log file information for a given url

        @url            URL where this exploit occured
        @module         Module/ActiveX Control, ... that gets exploited
        @description    Description of the exploit
        @cve            CVE number (if available)
        """
        if not self.json_enabled:
            return

        self.data["exploits"].append({"url"         : self.fix(url),
                                      "module"      : module,
                                      "description" : description,
                                      "cve"         : cve,
                                      "data"        : data})

    def log_classifier(self, classifier, url, rule, tags):
        """
        Log classifiers matching for a given url

        @classifier     Classifier name
        @url            URL where the rule match occurred
        @rule           Rule name
        @tags           Rule tags
        """
        if not self.json_enabled:
            return

        item = {"classifier" : classifier,
                "url"        : self.fix(url),
                "rule"       : rule,
                "tags"       : tags}

        if item not in self.data["classifiers"]:
            self.data["classifiers"].append(item)

    def add_behavior(self, description = None, cve = None, method = "Dynamic Analysis"):
        if not self.json_enabled:
            return

        if not cve and not description:
            return

        self.data["behavior"].append({"description" : self.fix(description),
                                      "cve"         : self.fix(cve),
                                      "method"      : self.fix(method),
                                      "timestamp"   : str(datetime.datetime.now())})

    def add_behavior_warn(self, description = None, cve = None, method = "Dynamic Analysis"):
        if not self.json_enabled:
            return

        self.add_behavior(description, cve, method)

    def log_file(self, data, url = None, params = None):
        if not self.json_enabled:
            return

        self.data["files"].append(data)

    def export(self, basedir):
        if not self.json_enabled:
            return

        output = StringIO()
        json.dump(self.data, output, sort_keys = False, indent = 4)
        if log.ThugOpts.json_logging and log.ThugOpts.file_logging:
            logdir = os.path.join(basedir, "analysis", "json")
            log.ThugLogging.store_content(logdir, 'analysis.json', output.getvalue())

            m = Mapper(logdir)
            m.add_data(self.data)
            m.write_svg()

        self.cached_data = output

    def get_json_data(self, basedir):
        if self.cached_data:
            return self.cached_data.getvalue()

        return None



#!/usr/bin/env python

import bs4 as BeautifulSoup

from .HTMLElement import HTMLElement
from .attr_property import attr_property


class HTMLBodyElement(HTMLElement):
    def __init__(self, doc, tag):
        HTMLElement.__init__(self, doc, tag)

    background      = attr_property("background")
    bgColor         = attr_property("bgcolor")
    link            = attr_property("link")
    aLink           = attr_property("alink")
    vLink           = attr_property("vlink")
    text            = attr_property("text")

    def getInnerHTML(self):
        html = unicode()

        for tag in self.tag.contents:
            html += unicode(tag)

        return html

    def setInnerHTML(self, html):
        self.tag.clear()

        for node in BeautifulSoup.BeautifulSoup(html, "html.parser").contents:
            self.tag.append(node)

    innerHTML = property(getInnerHTML, setInnerHTML)

    def __repr__(self):
        return "<HTMLBodyElement at 0x%08X>" % (id(self), )

    def __str__(self):
        body = self.doc.find('body')
        return str(body if body else self.doc)

    def __unicode__(self):
        body = self.doc.find('body')
        return unicode(body if body else self.doc)



#!/usr/bin/env python
#
# HoneyAgent.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA


import os
import base64
import requests
import tempfile
import logging

log = logging.getLogger("Thug")

try:
    import configparser as ConfigParser
except ImportError:
    import ConfigParser


class HoneyAgent(object):
    def __init__(self):
        self.enabled = True
        self.opts    = dict()

        self.__init_config()

    def __init_config(self):
        config = ConfigParser.ConfigParser()

        conf_file = os.path.join(log.configuration_path, 'honeyagent.conf')

        if not os.path.isfile(conf_file):
            self.enabled = False
            return

        config.read(conf_file)

        for option in config.options('HoneyAgent'):
            self.opts[option] = config.get('HoneyAgent', option)

    def save_report(self, response, basedir, sample):
        log_dir  = os.path.join(basedir, 'analysis', 'honeyagent')
        log.ThugLogging.log_honeyagent(log_dir, sample, response.text)

    def save_dropped(self, response, basedir, sample):
        data = response.json()

        result = data.get("result", None)
        if result is None:
            return None

        files = result.get("files", None)
        if files is None:
            return result

        md5 = sample['md5']
        log_dir = os.path.join(basedir, 'analysis', 'honeyagent', 'dropped')

        for filename in files.keys():
            log.warning("[HoneyAgent][%s] Dropped sample %s", md5, os.path.basename(filename))
            data = base64.b64decode(files[filename])
            log.ThugLogging.store_content(log_dir, os.path.basename(filename), data)
            log.ThugLogging.log_file(data)

        return result

    def dump_yara_analysis(self, result, sample):
        yara = result.get("yara", None)
        if yara is None:
            return

        md5 = sample['md5']

        for key in yara.keys():
            for v in yara[key]:
                log.warning("[HoneyAgent][%s] Yara %s rule %s match", md5, key, v['rule'])

    def submit(self, data, sample, params):
        md5    = sample['md5']
        sample = os.path.join(tempfile.gettempdir(), md5)

        with open(sample, "wb") as fd:
            fd.write(data)

        files    = {'file'  : (md5, open(sample, "rb"))}
        response = requests.post(self.opts["scanurl"], files = files, params = params)

        if response.ok:
            log.warning("[HoneyAgent][%s] Sample submitted", md5)

        os.remove(sample)
        return response

    def analyze(self, data, sample, basedir, params):
        if not self.enabled:
            return

        if not log.ThugOpts.honeyagent:
            return

        if params is None:
            params = dict()

        response = self.submit(data, sample, params)
        self.save_report(response, basedir, sample)
        result = self.save_dropped(response, basedir, sample)
        if result:
            self.dump_yara_analysis(result, sample)



#!/usr/bin/env python
#
# Shellcode.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA

import logging
import traceback
import six
import pylibemu
from .Debugger import Debugger

log = logging.getLogger("Thug")


class Shellcode(object):
    emu = pylibemu.Emulator(enable_hooks = False)

    def __init__(self, window, ctxt, ast, script):
        self.window  = window
        self.script  = script
        self.ctxt    = ctxt
        self.ast     = ast
        self.offsets = set()

    def check_URLDownloadToFile(self, emu):
        profile = emu.emu_profile_output

        while True:
            offset = profile.find('URLDownloadToFile')
            if offset < 0:
                break

            profile = profile[offset:]

            p = profile.split(';')
            if len(p) < 2:
                profile = profile[1:]
                continue

            p = p[1].split('"')
            if len(p) < 3:
                profile = profile[1:]
                continue

            url = p[1]
            if url in log.ThugLogging.shellcode_urls:
                return

            try:
                self.window._navigator.fetch(p[1], redirect_type = "Found URLDownloadToFile")
                log.ThugLogging.shellcode_urls.add(url)
            except:  # pylint:disable=bare-except
                pass

            profile = profile[1:]

    def search_url(self, sc):
        offset = sc.find('http')

        if offset > 0:
            url = sc[offset:].split()[0]
            if url.endswith("'") or url.endswith('"'):
                url = url[:-1]

            if url in log.ThugLogging.shellcode_urls:
                return

            log.info('[Shellcode Analysis] URL Detected: %s', url)

            try:
                self.window._navigator.fetch(url, redirect_type = "URL found")
                log.ThugLogging.shellcode_urls.add(url)
            except:  # pylint:disable=bare-except
                pass

    def run(self):
        with Debugger() as dbg:
            dbg._context = self.ctxt
            # dbg.debugBreak()

            try:
                result = self.ctxt.eval(self.script)
            except (UnicodeDecodeError, TypeError):
                try:
                    enc = log.Encoding.detect(self.script)
                    result = self.ctxt.eval(self.script.decode(enc['encoding']))
                except:  # pylint:disable=bare-except
                    log.ThugLogging.log_warning(traceback.format_exc())
                    return None
            except:  # pylint:disable=bare-except
                log.ThugLogging.log_warning(traceback.format_exc())
                return None

            for name in (n for n in self.ast.names if n in self.ctxt.locals):
                s = self.ctxt.locals[name]

                if not s:
                    continue

                if not isinstance(s, six.string_types):
                    continue

                # log.debug("[Shellcode] Testing variable: %s", name)
                self.emu.run(s)

                if self.emu.emu_profile_output:
                    log.ThugLogging.add_shellcode_snippet(self.emu.emu_profile_output, 'Assembly', 'Shellcode')
                    log.warning("[Shellcode Profile]\n\n%s", self.emu.emu_profile_output)
                    self.check_URLDownloadToFile(self.emu)

                self.emu.free()
                self.search_url(s)

        return result



#!/usr/bin/env python

from .HTMLElement import HTMLElement
from .attr_property import attr_property


class HTMLQuoteElement(HTMLElement):
    def __init__(self, doc, tag):
        HTMLElement.__init__(self, doc, tag)

    cite            = attr_property("cite")
